

//  ######
//  AI Builder v2.0
//  ExtensionFilter class.
//  By Richard John Sproston.
//  E-mail richard.sproston@gmail.com || lucky_jackpot@hotmail.com
//  Copyright () 2004/2005, Richard John Sproston.
//  All Rights Reserved.
//  Sept-2004
//  ######


package bin;


    // Import system class libraries for application
  import javax.swing.filechooser.FileFilter;
  import java.io.File;
  import java.io.*;


/**
 * Extension Filter Class
 * <p>
 * Set-up an extension filter for files, when prompted with "Save" dialogs.
 * </p>
 *
 * @author Richard John Sproston
 * [ <a href="mailto:richard.sproston@gmail.com">richard.sproston@gmail.com</a> ||
 * <a href="mailto:lucky_jackpot@hotmail.com">lucky_jackpot@hotmail.com</a> ]
 *
 * @version 0.3 - Initial Release
 *
 */
public class ExtensionFilter extends FileFilter {


//=========================================================


    //////////////////////////
    //  INSTANCE VARIABLES  //
    //////////////////////////

    private String extension;
    private String description;


//=========================================================


    /////////////////////////
    //  CONSTRUCTORS  //
    /////////////////////////

    /**
     * Default Constructor for class, with no parameters.
     */
    public ExtensionFilter()
    {}


    /**
     * Default Constructor for class, with 2 parameters.
     *
     * @param ext This is used to set a String data-type
     *  as a FileFilter extension.
     * @param desc This is used to provide a String description
     *  of the file extension.
     *
     */
    public ExtensionFilter (String ext, String desc) {
        extension = ext.toLowerCase();
        description = desc;
    }


//=========================================================


    ///////////////
    //  METHODS  //
    ///////////////


    /**
     * This method returns a boolean value, depending on whether the "file"
     * is successfully accepted by the application as being of the correct
     * data-type.
     *
     * @param file The File object to be checked for a comparison.
     *
     * @return The boolean value returned, depending on whether the "file"
     *  to be accepted is a directory or whether the file ends with the
     *  appropriate extension.
     */
    public boolean accept (File file) {
        return (
            file.isDirectory() ||
            file.getName().toLowerCase().endsWith (extension)
        );
    }


    /**
     * This method returns the String description for use in
     * manipulating the constructor's value for this variable.
     *
     * @return The String value/content of the variable, for manipulation
     *              in the class's constructor (2 params).
     *
     */
    public String getDescription() {
        return description;
    }


//=========================================================


}   /* End Of File  */