

//  ######
//  AI Builder v2.0.3
//  GUIFramework class.
//  By Richard John Sproston.
//  E-mail richard.sproston@gmail.com || lucky_jackpot@hotmail.com
//  Copyright () 2004, Richard John Sproston.
//  All Rights Reserved.
//  Sept-2004
//  ######


package bin;


    // Import required system libraries...
  import java.awt.*;
  import java.awt.event.*;
  import java.awt.event.ActionEvent;
  import java.awt.event.ActionListener;
  import javax.swing.*;
  import javax.swing.event.*;
  import javax.swing.border.*;
  import java.io.*;
  import java.io.File;
  import java.util.Properties;



/**
 * GUIFramework Class.
 * <p>
 * Constructs the visual GUI environment for the user to interact with.
 * </p>
 *
 * <br />
 * <br />
 * <br />
 *
 * version 0.3 - Initial Release <br />
 * <br />
 * <br />
 *
 * v0.4 - UPDATE: <br />
 *              Modified the"Save File" method, updating so the file is saved
 *              to the correct directory, rather than simply relative to the
 *              program.  Corresponding alterations made to
 *              <code>DataRecordOperations</code> class's "writeFile" method.
 * <br />
 * <br />
 *
 * v0.5 - UPDATE: <br />
 *              Modified the "Generate File" code to include only <b>basic</b>
 *              error checking.  AI Builder still doesn't handle input
 *              sanitization - maybe in a future release...  ;)
 * <br />
 * <br />
 *
 * v0.6 - MODIFICATION: <br />
 *              Implemented feedback suggestions (Darth Slaw) of retaining
 *              text-area input values, rather than clearing them
 *              (a pain if you accidentally de-select a "Use?" checkbox.
 *              Don't really know why I made the decision to do that the first
 *              time around...).
 *              <br /><br />
 *              Made some typing error adjustments.
 *              <br /><br />
 *              Added a static method to allow update versioning easier to
 *              manage and maintain.
 *              <br /><br />
 *              Still no input sanitization - then again, I've made no
 *              promises as to this feature being included...
 * <br />
 * <br />
 *
 * v0.7 - MODIFICATION: <br />
 *              Implemented feedback suggestions (ZeqMacaw) about
 *              "User Interface" issues.
 *              Includes roll-over effects, user interaction and default
 *              enabled/disabled options.
 * <br />
 * <br />
 *
 * v0.8 - UPDATE: <br />
 *              Modified existing code to load/read resources from a central
 *              data repository.  Amended a few minor inconsistencies and
 *              created a more manageable "flag"-based system to calculate
 *              which AI features should be made available, based on the
 *              "AI Special Characteristics" (applicable to specific types
 *              of AI).
 * <br />
 * <br />
 *
 * v0.9 - UPDATE: <br />
 *              Implemented an update for storing several
 *              "user-set preferences".
 *              Added a <code> WindowHandler </code> inner-class to monitor
 *              when application was closing and to store screen location
 *              co-ordinates for next start-up.
 *
 *
 * @author Richard John Sproston
 *  [ <a href="mailto:richard.sproston@gmail.com">richard.sproston@gmail.com</a> ||
 *  <a href="mailto:lucky_jackpot@hotmail.com">lucky_jackpot@hotmail.com</a> ]
 *
 * @version 0.9 Added WindowHandler inner-class to manage the saving of user
 *              preferences when closing the application.
 *
 */
public class GUIFramework extends JFrame {


//=========================================================


    //////////////////////////
    //  INSTANCE VARIABLES  //
    //////////////////////////


        /** CORE [ROOT] Panel that acts as a container
            for ALL the program's components */
    private JPanel masterPanel = new JPanel (new BorderLayout() );


    // SUB-ROOT major position panels
        /** JPanel for managing the top panel */
    private JPanel topPanel = new JPanel (new BorderLayout() );

        /** JPanel for manipulating the components
            in the bottom panel */
    private JPanel bottomPanel = new JPanel (new BorderLayout() );

        /** JPanel for manipulating/building the central panel
            with core components */
    private JPanel coreInterface = new JPanel();


    // Essential objects for repeated manipulation...
        /** The tabbed-pane interface to hold other existing components */
    private JTabbedPane tabsInterface = new JTabbedPane();
        /** The JPanel that contains the tabbed-pane
            GUI-related components */
    private JPanel tabbedPanel = new JPanel (new BorderLayout() );
        /** The application's status bar */
    private JLabel statusBar = new JLabel (" ", SwingConstants.LEFT);
        /** The text-area that provides additional information related
            to each instinct */
    private JTextArea expandedInfo = new JTextArea ("Expanded Information", 8, 50);
        /** This is where the generated AI File will be displayed */
    private JTextArea codeWindow = new JTextArea();
        /** This button generates an AI file, from the information provided */
    private JButton generateFile = new JButton ("Generate File...");
        /** This button will clear the existing codeWindow
            (when it has contents available) */
    private JButton clearCodeWindow = new JButton ("Clear Display");
        /** This button will save an AI file */
    private JButton saveFile = new JButton ("Save File");
        /** This button will open an existing/current AI file */
    private JButton openFile = new JButton ("Open File For Parsing");
        /** This button will attempt to parse a file whose content
            has been pre-loaded into the codeWindow */
    private JButton parseExternalFile = new JButton ("Parse Current Display...");


    // Miscellaneous String details for AI Builder to generate AI code...
        /** File header blurb */
    private String fileHeader = "";
        /** Game type: JK or MotS */
    private String gameType = "";
        /** Character alignment for AI actor */
    private String alignment = "";
        /** easy [.ai0], medium [.ai] or hard [.ai2] */
    private String difficultyFileNameExtension = "";
        /** Alignment, FOV etc. concatenated String line */
    private String miscTraits = "";
        /** Intended filename */
    private String fileName = "";


    // "initSetup" - Initial Setup Configurations for AI File */
        /** RadioButton group to handle selectable game types.
            Mutually exclusive set of options. */
    private JRadioButton [] rb_gameType = new JRadioButton [2];
        /** RadioButton group to handle the AI actor's alignment.
            Mutually exclusive set of options. */
    private JRadioButton [] rb_alignment = new JRadioButton [3];
        /** RadioButton group to handle the intended difficulty for the
            actor's AI.  Mutually exclusive set of options. */
    private JRadioButton [] rb_difficulty = new JRadioButton [3];
    private JTextField [] tf_miscTraits = new JTextField [6];
    private JLabel [] miscTraitsLabel = new JLabel [6];
    private JTextField tf_fileName = new JTextField();
    private JCheckBox [] cb_specialCharacterAI = new JCheckBox [6];


    // "jkSetup" - JK ORDINARY AI Attributes HERE !!! */
        /** Textfield array for <code>"BlindFire"</code>. 6 arguments */
    private JTextField [] tf_blindFireP = new JTextField [6];
        /** Textfield array for <code>"Crouch"</code>.
            Previously unknown/undocumented. 2 arguments */
    private JTextField [] tf_crouchP = new JTextField [2];
        /** Textfield array for <code>"CircleStrafe"</code>. 5 arguments */
    private JTextField [] tf_circleStrafeP = new JTextField [5];
        /** Textfield array for <code>"Dodge"</code>. 2 arguments */
    private JTextField [] tf_dodgeP = new JTextField [2];
        /** TextField array for <code>"Flee"</code>. 3 arguments */
    private JTextField [] tf_fleeP = new JTextField [3];
        /** Textfield array for <code>"Follow"</code>. 2-4 arguments */
    private JTextField [] tf_followP = new JTextField [4];
        /** Textfield array for <code>"HitAndRun"</code>. 3 arguments */
    private JTextField [] tf_hitAndRunP = new JTextField [2];
        /** Textfield array for <code>"Jump"</code>. 3 arguments */
    private JTextField [] tf_jumpP = new JTextField [3];
        /** Textfield array for <code>"Listen"</code>. 2 arguments */
    private JTextField [] tf_listenP = new JTextField [2];
        /** Textfield array for <code>"LobFire"</code>. 6 arguments */
    private JTextField [] tf_lobFireP = new JTextField [6];
        /** Textfield array for <code>"LookForTarget"</code>.
            Also applicable for TurretGun. 2 arguments */
    private JTextField [] tf_lookForTargetP = new JTextField [2];
        /** Checkbox for <code>"OpenDoors"</code>.  Boolean value. */
    private JCheckBox cb_openDoorsP =
    new JCheckBox ("Should the character's AI be able to open doors (Yes) ?");
        /** Textfield array for <code>"PrimaryFire"</code>. 10 arguments */
    private JTextField [] tf_primaryFireP = new JTextField [10];
        /** Textfield array for <code>"RandomTurn"</code>. 2 arguments */
    private JTextField [] tf_randomTurnP = new JTextField [2];
        /** Textfield array for <code>"Retreat"</code>. 4 arguments */
    private JTextField [] tf_retreatP = new JTextField [4];
        /** Checkbox for <code>"ReturnHome"</code>.  Boolean value. */
    private JCheckBox cb_returnHomeP =
    new JCheckBox ("Should the character's AI return home (Yes) ?");
        /** Textfield array for <code>"Roam"</code>. 2 arguments */
    private JTextField [] tf_roamP = new JTextField [2];
        /** Textfield array for <code>"SenseDanger"</code>. 2 arguments */
    private JTextField [] tf_senseDangerP = new JTextField [2];
        /** Textfield array for <code>"Talk"</code>. 2 arguments */
    private JTextField [] tf_talkP = new JTextField [2];
        /** Textfield array for <code>"Withdraw"</code>. 2 arguments */
    private JTextField [] tf_withdrawP = new JTextField [2];


    // "turretGun" - Turret Gun AI HERE !!! */
    	/** Textfield array for <code>"LookForTarget"</code>. 2 arguments
    		<br />
			NOTE: Name different *2P*, to signify a second batch of
			textfields for TurretGun! */
    private JTextField [] tf_lookForTarget2P = new JTextField [2];
    	/** Textfield array for <code>"TurretFire"</code>. 9 arguments */
    private JTextField [] tf_turretFireP = new JTextField [9];


    // "motsSetup" - MOTS ADDITIONAL AI Attributes HERE !!! */
    	/** Textfield array for <code>"Charge"</code>. 6 arguments
    		<br />
    		Previously undocumented instinct - a new discovery. */
    private JTextField [] tf_chargeP = new JTextField [6];
    	/** Textfield array for <code>"Leap"</code>. 6 arguments  */
    private JTextField [] tf_leapP = new JTextField [6];
    	/** Textfield array for <code>"LookForOpposingTarget"</code>.
    		2 arguments */
    private JTextField [] tf_lookForOpposingTargetP = new JTextField [2];


    // "jediSpecial" - Special Jedi AI Attributes */
    	/** Textfield array for <code>"ForcePowers"</code>. 13 arguments */
    private JTextField [] tf_forcePowersP = new JTextField [13];
    	/** Textfield array for <code>"SaberFighting"</code>. 13 arguments */
    private JTextField [] tf_saberFightingP = new JTextField [13];
    	/** Textfield array for <code>"SpecialAttack"</code>. 9 arguments */
    private JTextField [] tf_specialAttackP = new JTextField [9];



    // ButtonGroups  */
    	/** A radio-button group for the AI character's alignment. */
    private ButtonGroup alignmentState;
    	/** A radio-button group for the AI character's game-type. */
    private ButtonGroup gameTypeState;
    	/** A radio-button group for the AI level of difficulty in-game. */
    private ButtonGroup difficultyState;


    // INNER-CLASS REFERENCES  */
        /** Inner-class listener for all check-box related GUI components. */
    private CheckBoxListener checkBoxSetting = new CheckBoxListener();
        /** Inner-class listener for all button related GUI components. */
    private ButtonProcessor buttonListener = new ButtonProcessor();
        /** Inner-class listener for all radio-button related GUI components. */
    private RadioButtonListener radioButtonSetting = new RadioButtonListener();


        /** Array of checkboxes to manage whether or not an instinct
            should be used (and therefore checked for validation, when
            parsing is requested).
            Check to see which instincts to parse. */
    private JCheckBox [] useThisInstinct = new JCheckBox [26];


    // Database resource loading objects */
        /** The properties object that loads the input stream, referring
            to sirSource's source. */
    private Properties resources = new Properties();
        /** Stream-Input-Reader Source, required to load an input
            stream, referring to the resource file. */
    private InputStream sirSource;


	// Application information */
        /** static String - application information */
    private static String INFO;
        /** The top-left corner co-ordinate position for the
            application window's frame. */
    private static String WINDOW_X_KEY = "top_left_coord";
        /** The bottom-left corner co-ordinate for the
            application window's frame. */
    private static String WINDOW_Y_KEY = "btm_left_coord";
        /** The top-right corner co-ordinate for the
            application window's frame. */
    private static String WINDOW_WIDTH = "top_right_coord";
        /** The bottom-right corner co-ordinate for the
            application window's frame. */
    private static String WINDOW_HEIGHT = "btm_right_coord";



//=========================================================


    //////////////////////
    //  STATIC METHODS  //
    //////////////////////


    /**
     * Set static string information, regarding both
     * version and application information.
     *
     */
    static {
        INFO = ": A JK+MotS AI File Generator ";
    }


//=========================================================


    ////////////////////
    //  CONSTRUCTORS  //
    ////////////////////


    /**
     * Default Constructor for class with no parameters.
     */
    public GUIFramework() {
        super (
            GlobalConstants.TITLE +
            INFO +
            GlobalConstants.VERSION
            );

        Toolkit window = this.getToolkit();
        Dimension screenSize = window.getScreenSize();


        try {
            //sirSource = getClass().getResourceAsStream (
            //    GlobalConstants.RESOURCE);
            sirSource = new FileInputStream (GlobalConstants.RESOURCE);
            resources.load (sirSource);
        }
        catch (IOException IO) {
            JOptionPane.showMessageDialog (null,
                "A critical application resource could not be located. \n" +
                "The application cannot function without it." +
                "\n\n" +
                "Could not reach: " + resources.toString() +
                "\n\n" +
                "Aborting . . . ",
                "Database Resource Reference File Missing",
                JOptionPane.ERROR_MESSAGE);
            System.gc();
            System.exit (1);
        }

        this.setBounds (
            Integer.parseInt (resources.getProperty ("top_left_coord")),  // 0
            Integer.parseInt (resources.getProperty ("btm_left_coord")),  // 0
            Integer.parseInt (resources.getProperty ("top_right_coord")), // 800
            Integer.parseInt (resources.getProperty ("btm_right_coord"))  // 600
        );

        JSplitPane masterSplitter = new JSplitPane (
            JSplitPane.VERTICAL_SPLIT, buildCoreInterface(), getBottomPanel() );

        masterSplitter.setDividerLocation (400);

        //masterPanel.add (buildCoreInterface(), BorderLayout.CENTER);
        //masterPanel.add (getBottomPanel(), BorderLayout.SOUTH);
        masterPanel.add (masterSplitter, BorderLayout.CENTER);

        getContentPane().add (masterPanel);

        //setDefaultCloseOperation (EXIT_ON_CLOSE);
        this.addWindowListener (new WindowHandler() );
    }


//=========================================================


    ////////////////////////
    //  ACCESS MODIFIERS  //
    ////////////////////////


    /**
     * Sets the game-type for the intended AI file.
     *
     * @param gameType The game-type for the AI (either JK or MotS).
     *
     */
    public void setGameType (String gameType) {
        this.gameType = gameType;
    }


    /**
     * Sets the alignment of the AI for participation in-game.
     *
     * @param alignment The alignment to use for the AI character.
     *
     */
    public void setAlignment (String alignment) {
        this.alignment = alignment;
    }


    /**
     * Sets the difficulty (and appropriate filename extension)
     * for the AI file's character.
     *
     * @param difficultyFileNameExtension The difficulty level
     *  (and therefore the appropriate extension as well).
     *
     */
    public void setDifficultyFileNameExtension (
    String difficultyFileNameExtension) {
        this.difficultyFileNameExtension = difficultyFileNameExtension;
    }


    /**
     * Sets the miscellaneous traits of the AI character
     * (those that comprise the "AI setup line").
     *
     * @param miscTraits The (mandatory) String of miscellanous AI traits
     *  that form the AI Setup line.
     *
     */
    public void setMiscTraits (String miscTraits) {
        this.miscTraits = miscTraits;
    }


    /**
     * Sets the intended name of the AI file
     * (ready for saving + displaying in preview window).
     *
     * @param fileName The name of the intended AI file.
     *
     */
    public void setFileName (String fileName) {
        this.fileName = fileName;
    }



    public void setStatusBarText (String statusText) {
        statusBar.setText (" " + statusText);
    }


//=========================================================


    /////////////////////////
    //  ACCESS RETRIEVERS  //
    /////////////////////////


    /**
     * This method retrieves the "fileHeader" String.
     * This String concatenates the header text commentary, including
     * game-type, file name, AI difficulty level and the "AI setup line".
     * <p>
     * Technically, more of a "setter" method than an
     * "accessor/retriever" method.
     * </p>
     *
     * @return The concatenated fileHeader String.
     *
     */
    private String getFileHeader() {
        fileHeader =
            "##\n" +
            "# " + getGameType() + " AI File \n" +
            "#\n" +
            "# " + getFileName().toUpperCase() +
                getDifficultyFileNameExtension() + "\n" +
            "#\n" +
            "# File Generated by \"" +
            GlobalConstants.TITLE + " " + GlobalConstants.VERSION +
            "\" by Jackpot \n" +
            "# [RJS - lucky_jackpot@hotmail.com] \n" +
            "#\n" +
            "##\n" +
            "\n" + getMiscTraits() + "\n\n" +
            "# instinct \t\t args \n";
        return fileHeader;
    }


    /**
     * Gets the game type used (either JK or MotS).
     *
     * @return The intended game-type for the AI to function with.
     *
     */
    private String getGameType() {
        return gameType;
    }


    /**
     * Gets the alignment of the AI
     * (-1 = enemy ; 0.0 = neutral ; 1.0 = ally)
     *
     * @return The intended alignment setting for the AI to function with.
     */
    private String getAlignment() {
        return alignment;
    }


    /**
     * Gets the difficulty level for the AI
     * (ai0 = easy ; ai = medium ; ai2 = hard).
     * This method also uses the value to set the file extension for the
     * intended file upon saving.
     *
     * @return The intended difficulty level for the JK/MotS engine to
     *  apply the AI to.
     *
     */
    private String getDifficultyFileNameExtension() {
        return difficultyFileNameExtension;
    }


    /**
     * This method retrieves the various miscellaneous AI traits that form
     * the "AI setup line".  This method has a few quirks depending on what
     * options have been provided for the program to work with in the opening
     * AI Setup Panel.
     *
     * @return The mandatory AI setup line for the AI character file.
     *
     */
    private String getMiscTraits() {
        miscTraits = "alignment=" + getAlignment();
        miscTraits += ", rank=" + tf_miscTraits[0].getText();
        miscTraits += ", fov=" + tf_miscTraits[1].getText();

        //if (cb_specialCharacterAI[1].isSelected() ) {
        //    // do nothing...
        //}
        //else
        if (cb_specialCharacterAI[3].isSelected() ||
        	cb_specialCharacterAI[4].isSelected() ) {
            	miscTraits += ", maxstep=0.0";
        }
        else {    // ...else add maxstep value to miscTraits String...
            miscTraits += ", maxstep=" + tf_miscTraits[2].getText();
        }

        miscTraits += ", sightdist=" + tf_miscTraits[3].getText();
        miscTraits += ", heardist=" + tf_miscTraits[4].getText();

        /*  Research shows that passive/neutral AI and (Dark) Jedi do NOT
            have an "accuracy" value.  The former is understandable, as they
            do not attack the player.  The latter, meanwhile, is possibly
            because they have a "SaberFighting" instinct to handle attacking,
            so they don't need an accuracy value? ... */
        if (rb_alignment[1].isSelected() ||
            cb_specialCharacterAI[2].isSelected() ||
        	cb_specialCharacterAI[3].isSelected() )
        		// return a value without the AI "accuracy=" setup line...
            return miscTraits;
        else    // ... else append to the String we already have.
            miscTraits += ", accuracy=" + tf_miscTraits[5].getText();

        return miscTraits;
    }


    /**
     * Get the intended file name.
     *
     * @return The intended file name for the AI file.
     *
     */
    private String getFileName() {
        fileName = tf_fileName.getText();
        return fileName;
    }


//=========================================================


    ///////////////
    //  METHODS  //
    ///////////////


    /**
     * Build the bottom panel and set various applicable attributes.
     * This includes the status bar and the "expanded info" sections
     * of the program.
     *
     * @return The bottom panel for the AI Builder program.
     *
     */
    public JPanel getBottomPanel() {
        expandedInfo.setBackground (Color.lightGray);
        expandedInfo.setForeground (Color.red);   // red || white ?
        expandedInfo.setFont (new Font ("Tahoma", Font.BOLD, 14) ); // 12,14,16
        expandedInfo.setEditable (false);
        expandedInfo.setLineWrap (true);
        expandedInfo.setWrapStyleWord (true);

            // Provide a margin-inset around this text-area
        expandedInfo.setBorder (
            BorderFactory.createEtchedBorder (EtchedBorder.LOWERED) );

            // Add a scroll-pane for the expanded-info text-area container
        JScrollPane expandedInfoScroller = new JScrollPane (expandedInfo);

        statusBar.setBorder (
            BorderFactory.createLoweredBevelBorder() );

        bottomPanel.add (expandedInfoScroller, BorderLayout.CENTER);
        bottomPanel.add (statusBar, BorderLayout.SOUTH);

        return bottomPanel;
    }


    /**
     * This is the central "builder" part of the program
     * (that branches off to build other parts) but every other
     * "build_[part]" method stems from this root method
     * (such as "buildCodeWindowAndOperations and
     * "buildTabsInterface" methods).
     *
     * @return The core interface JPanel for display.
     *
     */
    public JPanel buildCoreInterface() {
        //coreInterface.setLayout (new GridLayout (1, 2, 2, 2) );
        //coreInterface.add (buildTabsInterface() );
        //coreInterface.add (buildCodeWindowAndOperations() );

        JSplitPane splitPane = new JSplitPane (
            JSplitPane.HORIZONTAL_SPLIT,
            buildTabsInterface(),
            buildCodeWindowAndOperations() );

        splitPane.setDividerLocation (400);

        coreInterface.setLayout (new BorderLayout() );
        coreInterface.add (splitPane, BorderLayout.CENTER);

        return coreInterface;
    }


    /**
     * This builds the "right-hand-side" components of AI Builder,
     * including the code window and accompanying buttons and their
     * various operations.
     *
     * @return The code panel, including buttons
     *  (and the different action results of clicking on them).
     *
     */
    public JPanel buildCodeWindowAndOperations() {
        JPanel codePanel = new JPanel (new BorderLayout() );
        JPanel tempPanel = new JPanel (new GridLayout (1, 2) );

            // Set shortcut keys for buttons, as accessible by
            // "ALT + <keyboard_letter>"
        generateFile.setMnemonic (KeyEvent.VK_G);
        clearCodeWindow.setMnemonic (KeyEvent.VK_C);
        saveFile.setMnemonic (KeyEvent.VK_S);
        openFile.setMnemonic (KeyEvent.VK_O);
        parseExternalFile.setMnemonic (KeyEvent.VK_P);

            // Add listeners to all GUI components, so events
            // are registered when buttons are clicked.
        generateFile.addActionListener (buttonListener);
        clearCodeWindow.addActionListener (buttonListener);
        saveFile.addActionListener (buttonListener);
        openFile.addActionListener (buttonListener);
        parseExternalFile.addActionListener (buttonListener);

        openFile.setEnabled (true);
        parseExternalFile.setEnabled (true);

        openFile.setToolTipText ("Open an existing AI file");
        parseExternalFile.setToolTipText ("Parse an imported AI file");

        tempPanel.add (generateFile);
        codePanel.add (tempPanel, BorderLayout.NORTH);

        codeWindow.setToolTipText (
            "Preview of generated/imported AI File...");
        JScrollPane scrollCodeWindow = new JScrollPane (codeWindow);
        codePanel.add (scrollCodeWindow, BorderLayout.CENTER);

        tempPanel = new JPanel (new GridLayout (2, 2) );
        tempPanel.add (clearCodeWindow);
        tempPanel.add (parseExternalFile);
        tempPanel.add (openFile);
        tempPanel.add (saveFile);
        codePanel.add (tempPanel, BorderLayout.SOUTH);

        return codePanel;
    }


    /**
     * This monster method builds the tabbed interface with all their
     * accompanying components, such as scrollbars, checkboxes and the
     * actual tabs themselves.  Also sets up checkbox listeners to
     * register actions and respond accordingly.
     *
     * @return The tabbed panel visual components for display on-screen.
     *
     */
    public JPanel buildTabsInterface() {
        tabsInterface =
            new JTabbedPane (JTabbedPane.TOP, JTabbedPane.SCROLL_TAB_LAYOUT);

        JPanel initSetup = new JPanel (new GridLayout (6, 1, 0, 0) );
        JPanel jkSetup = new JPanel (new GridLayout (20, 1, 0, 0) );
        JPanel turretSetup = new JPanel (new GridLayout (6, 1, 0, 0) );
        JPanel motsSetup = new JPanel (new GridLayout (6, 1, 0, 0) );
        JPanel jediSpecialSetup = new JPanel (new GridLayout (6, 1, 0, 0) );

        JPanel temp = new JPanel();


    /******************** Build "Set-up" Tab Panel ********************/

        temp = new JPanel (new GridLayout (1, 2, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Select Game Type") );
        gameTypeState = new ButtonGroup();
        for (int i=0 ; i<rb_gameType.length ; i++) {
            rb_gameType[i] = new JRadioButton();
            gameTypeState.add (rb_gameType[i] );
            rb_gameType[i].addItemListener (radioButtonSetting);
            rb_gameType[i].setPreferredSize (new Dimension (30, 20) );
            temp.add (rb_gameType[i] );
        }
        rb_gameType[0].setToolTipText ("JK");
        rb_gameType[0].setText ("JK");
/*            "<html>" +
            "<body bgcolor='white'>" +
            "<h1> JK </h1>" +
            "</body>" +
            "</html>"
        );
*/
        rb_gameType[1].setToolTipText ("MotS");
        rb_gameType[1].setText ("MotS");
        initSetup.add (temp);

        temp = new JPanel (new GridLayout (1, 3, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "AI Character Alignment") );
        alignmentState = new ButtonGroup();
        for (int i=0 ; i<rb_alignment.length ; i++) {
            rb_alignment[i] = new JRadioButton();
            alignmentState.add (rb_alignment[i] );
            rb_alignment[i].addItemListener (radioButtonSetting);
            rb_alignment[i].setPreferredSize (new Dimension (30, 20) );
            temp.add (rb_alignment[i] );
        }
        rb_alignment[0].setText ("Enemy/Nemesis");
        rb_alignment[1].setText ("Neutral");
        rb_alignment[2].setText ("Ally/Friend");
        initSetup.add (temp);

        temp = new JPanel (new GridLayout (1, 3, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "AI Difficulty") );
        difficultyState = new ButtonGroup();
        for (int i=0 ; i<rb_difficulty.length ; i++) {
            rb_difficulty[i] = new JRadioButton();
            difficultyState.add (rb_difficulty[i] );
            rb_difficulty[i].addItemListener (radioButtonSetting);
            rb_difficulty[i].setPreferredSize (new Dimension (30, 20) );
            temp.add (rb_difficulty[i] );
        }
        rb_difficulty[0].setText ("Easy");
        rb_difficulty[1].setText ("Medium");
        rb_difficulty[2].setText ("Hard");
        initSetup.add (temp);

        temp = new JPanel (new GridLayout (6, 2, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Misc. AI Traits") );
        for (int i=0 ; i<tf_miscTraits.length ; i++) {
            tf_miscTraits[i] = new JTextField();
            tf_miscTraits[i].setColumns (10);
            miscTraitsLabel[i] = new JLabel();
            miscTraitsLabel[i].setText (resources.getProperty ("label" + i));
            tf_miscTraits[i].setToolTipText (
				resources.getProperty ("tooltip" + i));
            temp.add (miscTraitsLabel[i] );
            temp.add (tf_miscTraits[i] );
        }
        tf_miscTraits[0].setText ("0.5");
        tf_miscTraits[0].setEnabled (false);
        initSetup.add (temp);


        temp = new JPanel (new GridLayout (6, 2, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "File Name Details") );
        JLabel fileNameDetailsLabel [] = new JLabel [5];
        for (int i=0 ; i<fileNameDetailsLabel.length ; i++) {
            if (i==2) {
                temp.add (new JLabel (
                    resources.getProperty ("filenameInfo"),
                    SwingConstants.LEFT) );
            }
            else
            if (i == 3) {
                tf_fileName.setToolTipText (
                    resources.getProperty ("filenameTT"));
                tf_fileName.setColumns (10);
                temp.add (tf_fileName);
            }
            else {
                    // filler padding
                temp.add (new JLabel(" "));
            }
        }
        initSetup.add (temp);
/*
        GridBagConstraints pos = new GridBagConstraints();
        temp = new JPanel (new GridBagLayout() );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(),"File Name Details") );
        pos.gridy = 0;      pos.gridx = 0;
        temp.add (new JLabel (
            "Intended File Name (minus extension): ",
            SwingConstants.LEFT),
            pos);
        pos.gridy = 0;      pos.gridx = 1;
        tf_fileName.setColumns (12);
        temp.add (tf_fileName, pos);
        initSetup.add (temp);
*/

        temp = new JPanel (new GridLayout (3, 2, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Special Character AI") );
        for (int i=0 ; i<cb_specialCharacterAI.length ; i++) {
            cb_specialCharacterAI[i] = new JCheckBox();
            cb_specialCharacterAI[i].addActionListener (checkBoxSetting);

            //    // DO NOT ENABLE UNTIL A GAMETYPE HAS BEEN SELECTED !!!
            //cb_specialCharacterAI[i].setEnabled (true);
            cb_specialCharacterAI[i].setText (
                resources.getProperty ("specCharAI" + i));
            cb_specialCharacterAI[i].setToolTipText (
                resources.getProperty ("specCharTT" + i));
            temp.add(cb_specialCharacterAI[i] );
        }

        initSetup.add (temp);
    // END SETUP PANEL



    /******************** Build "JK ONLY" Tab Panel ********************/

            // Temporary sub-container to split and
            // place the "Use?" checkbox...
        JPanel subPanel = new JPanel();

        temp = new JPanel (new GridLayout (2 ,1, 2, 2) );   // 1 ,6, 2, 2)
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Blind Fire Parameters") );
        temp.add ( (useThisInstinct[0] = new JCheckBox() ) );
            useThisInstinct[0].setText ("Use?");
            useThisInstinct[0].setToolTipText ("Use this Instinct?");
            useThisInstinct[0].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_blindFireP.length ; i++) {
            tf_blindFireP[i] = new JTextField();
            tf_blindFireP[i].setEnabled (false);
            tf_blindFireP[i].setBackground (Color.lightGray);
            tf_blindFireP[i].setPreferredSize (new Dimension (20, 20) );
            tf_blindFireP[i].setToolTipText (
                resources.getProperty ("BlindFire" + i));
            subPanel.add (tf_blindFireP[i] );
        }
            // Balancing filler content -
            // sets spacing of components to be more consistent
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Circle / Strafe Parameters") );
        temp.add ( (useThisInstinct[1] = new JCheckBox() ) );
            useThisInstinct[1].setText ("Use?");
            useThisInstinct[1].setToolTipText ("Use this Instinct?");
            useThisInstinct[1].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_circleStrafeP.length ; i++) {
            tf_circleStrafeP[i] = new JTextField();
            tf_circleStrafeP[i].setEnabled (false);
            tf_circleStrafeP[i].setBackground (Color.lightGray);
            tf_circleStrafeP[i].setToolTipText (
                resources.getProperty ("CircleStrafe" + i));
            subPanel.add (tf_circleStrafeP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
            /* W00tage - I FOUND THIS UNDOCUMENTED INSTINCT !!! */
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Crouch") );
        temp.add ( (useThisInstinct[2] = new JCheckBox() ) );
            useThisInstinct[2].setText ("Use?");
            useThisInstinct[2].setToolTipText ("Use this Instinct?");
            useThisInstinct[2].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        JLabel [] filler = new JLabel[2];
        JLabel [] filler2 = new JLabel[2];
        for (int i=0 ; i<tf_crouchP.length ; i++) {
            tf_crouchP[i] = new JTextField();
            tf_crouchP[i].setEnabled (false);
            tf_crouchP[i].setBackground (Color.lightGray);
            tf_crouchP[i].setToolTipText (
                resources.getProperty ("Crouch" + i));
            subPanel.add (tf_crouchP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Dodge") );
        temp.add ( (useThisInstinct[3] = new JCheckBox() ) );
            useThisInstinct[3].setText ("Use?");
            useThisInstinct[3].setToolTipText ("Use this Instinct?");
            useThisInstinct[3].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_dodgeP.length ; i++) {
            tf_dodgeP[i] = new JTextField();
            tf_dodgeP[i].setEnabled (false);
            tf_dodgeP[i].setBackground (Color.lightGray);
            tf_dodgeP[i].setToolTipText (
                resources.getProperty ("Dodge" + i));
            subPanel.add (tf_dodgeP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Flee") );
        temp.add ( (useThisInstinct[4] = new JCheckBox() ) );
            useThisInstinct[4].setText ("Use?");
            useThisInstinct[4].setToolTipText ("Use this Instinct?");
            useThisInstinct[4].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_fleeP.length ; i++ ) {
            tf_fleeP[i] = new JTextField();
            tf_fleeP[i].setEnabled (false);
            tf_fleeP[i].setBackground (Color.lightGray);
            tf_fleeP[i].setToolTipText (
                resources.getProperty ("Flee" + i));
            subPanel.add (tf_fleeP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Follow") );
        temp.add ( (useThisInstinct[5] = new JCheckBox() ) );
            useThisInstinct[5].setText ("Use?");
            useThisInstinct[5].setToolTipText ("Use this Instinct?");
            useThisInstinct[5].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_followP.length ; i++ ) {
            tf_followP[i] = new JTextField();
            tf_followP[i].setEnabled (false);
            tf_followP[i].setBackground (Color.lightGray);
            tf_followP[i].setToolTipText (
                resources.getProperty ("Follow0" + i));
            subPanel.add (tf_followP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Hit and Run") );
        temp.add ( (useThisInstinct[6] = new JCheckBox() ) );
            useThisInstinct[6].setText ("Use?");
            useThisInstinct[6].setToolTipText ("Use this Instinct?");
            useThisInstinct[6].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_hitAndRunP.length ; i++ ) {
            filler[i] = new JLabel (" ");
            tf_hitAndRunP[i] = new JTextField();
            tf_hitAndRunP[i].setEnabled (false);
            tf_hitAndRunP[i].setBackground (Color.lightGray);
            tf_hitAndRunP[i].setToolTipText (
                resources.getProperty ("HitAndRun" + i));
            subPanel.add (tf_hitAndRunP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Jump") );
        temp.add ( (useThisInstinct[7] = new JCheckBox() ) );
            useThisInstinct[7].setText ("Use?");
            useThisInstinct[7].setToolTipText ("Use this Instinct?");
            useThisInstinct[7].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_jumpP.length ; i++) {
            tf_jumpP[i] = new JTextField();
            tf_jumpP[i].setEnabled (false);
            tf_jumpP[i].setBackground (Color.lightGray);
            tf_jumpP[i].setToolTipText (
                resources.getProperty ("Jump" + i));
            subPanel.add (tf_jumpP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Listen") );
        temp.add ( (useThisInstinct[8] = new JCheckBox() ) );
            useThisInstinct[8].setText ("Use?");
            useThisInstinct[8].setToolTipText ("Use this Instinct?");
            useThisInstinct[8].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_listenP.length ; i++) {
            tf_listenP[i] = new JTextField();
            tf_listenP[i].setEnabled (false);
            tf_listenP[i].setBackground (Color.lightGray);
            tf_listenP[i].setToolTipText (
                resources.getProperty ("Listen" + i));
            subPanel.add (tf_listenP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Lob Fire") );
        temp.add ( (useThisInstinct[9] = new JCheckBox() ) );
            useThisInstinct[9].setText ("Use?");
            useThisInstinct[9].setToolTipText ("Use this Instinct?");
            useThisInstinct[9].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_lobFireP.length ; i++) {
            tf_lobFireP[i] = new JTextField();
            tf_lobFireP[i].setEnabled (false);
            tf_lobFireP[i].setBackground (Color.lightGray);
            tf_lobFireP[i].setToolTipText (
                resources.getProperty ("LobFire" + i));
            subPanel.add (tf_lobFireP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Look For Target") );
        temp.add ( (useThisInstinct[10] = new JCheckBox() ) );
            useThisInstinct[10].setText ("Use?");
            useThisInstinct[10].setToolTipText ("Use this Instinct?");
            useThisInstinct[10].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_lookForTargetP.length ; i++) {
            tf_lookForTargetP[i] = new JTextField();
            tf_lookForTargetP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_lookForTargetP[i].setEnabled (false);
            tf_lookForTargetP[i].setBackground (Color.lightGray);
            tf_lookForTargetP[i].setToolTipText (
                resources.getProperty ("LookForTarget" + i));
            subPanel.add (tf_lookForTargetP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (1, 1, 0, 0) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Open Doors") );
            cb_openDoorsP.setToolTipText (
                resources.getProperty ("OpenDoors0"));
            cb_openDoorsP.addActionListener (checkBoxSetting);
            temp.add (cb_openDoorsP);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Primary Fire") );
        temp.add ( (useThisInstinct[11] = new JCheckBox() ) );
            useThisInstinct[11].setText ("Use?");
            useThisInstinct[11].setToolTipText ("Use this Instinct?");
            useThisInstinct[11].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 5, 2, 2) );
        for (int i=0 ; i<tf_primaryFireP.length ; i++) {
            tf_primaryFireP[i] = new JTextField();
            tf_primaryFireP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_primaryFireP[i].setEnabled (false);
            tf_primaryFireP[i].setBackground (Color.lightGray);
            tf_primaryFireP[i].setToolTipText (
                resources.getProperty ("PrimaryFire" + i));
            subPanel.add (tf_primaryFireP[i] );
        }
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Random Turns") );
        temp.add ( (useThisInstinct[12] = new JCheckBox() ) );
            useThisInstinct[12].setText ("Use?");
            useThisInstinct[12].setToolTipText ("Use this Instinct?");
            useThisInstinct[12].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_randomTurnP.length ; i++) {
            tf_randomTurnP[i] = new JTextField();
            tf_randomTurnP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_randomTurnP[i].setEnabled (false);
            tf_randomTurnP[i].setBackground (Color.lightGray);
            tf_randomTurnP[i].setToolTipText (
                resources.getProperty ("RandomTurn" + i));
            subPanel.add (tf_randomTurnP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Retreat") );
        temp.add ( (useThisInstinct[13] = new JCheckBox() ) );
            useThisInstinct[13].setText ("Use?");
            useThisInstinct[13].setToolTipText ("Use this Instinct?");
            useThisInstinct[13].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_retreatP.length ; i++) {
            tf_retreatP[i] = new JTextField();
            tf_retreatP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_retreatP[i].setEnabled (false);
            tf_retreatP[i].setBackground (Color.lightGray);
            tf_retreatP[i].setToolTipText (
                resources.getProperty ("Retreat" + i));
            subPanel.add (tf_retreatP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (1, 1, 0, 0) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Return Home") );
            cb_returnHomeP.setToolTipText (
                resources.getProperty ("ReturnHome0"));
            cb_returnHomeP.addActionListener (checkBoxSetting);
            temp.add (cb_returnHomeP);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Roam") );
        temp.add ( (useThisInstinct[14] = new JCheckBox() ) );
            useThisInstinct[14].setText ("Use?");
            useThisInstinct[14].setToolTipText ("Use this Instinct?");
            useThisInstinct[14].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_roamP.length ; i++) {
            tf_roamP[i] = new JTextField();
            tf_roamP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_roamP[i].setEnabled (false);
            tf_roamP[i].setBackground (Color.lightGray);
            tf_roamP[i].setToolTipText (
                resources.getProperty ("Roam" + i));
            subPanel.add (tf_roamP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Sense Danger") );
        temp.add ( (useThisInstinct[15] = new JCheckBox() ) );
            useThisInstinct[15].setText ("Use?");
            useThisInstinct[15].setToolTipText ("Use this Instinct?");
            useThisInstinct[15].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_senseDangerP.length ; i++) {
            tf_senseDangerP[i] = new JTextField();
            tf_senseDangerP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_senseDangerP[i].setEnabled (false);
            tf_senseDangerP[i].setBackground (Color.lightGray);
            tf_senseDangerP[i].setToolTipText (
                resources.getProperty ("SenseDanger" + i));
            subPanel.add (tf_senseDangerP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Talk") );
        temp.add ( (useThisInstinct[16] = new JCheckBox() ) );
            useThisInstinct[16].setText ("Use?");
            useThisInstinct[16].setToolTipText ("Use this Instinct?");
            useThisInstinct[16].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_talkP.length ; i++) {
            tf_talkP[i] = new JTextField();
            tf_talkP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_talkP[i].setEnabled (false);
            tf_talkP[i].setBackground (Color.lightGray);
            tf_talkP[i].setToolTipText (
                resources.getProperty ("Talk" + i));
            subPanel.add (tf_talkP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Withdraw") );
        temp.add ( (useThisInstinct[17] = new JCheckBox() ) );
            useThisInstinct[17].setText ("Use?");
            useThisInstinct[17].setToolTipText ("Use this Instinct?");
            useThisInstinct[17].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_withdrawP.length ; i++) {
            tf_withdrawP[i] = new JTextField();
            tf_withdrawP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_withdrawP[i].setEnabled (false);
            tf_withdrawP[i].setBackground (Color.lightGray);
            tf_withdrawP[i].setToolTipText (
                resources.getProperty ("Withdraw" + i));
            subPanel.add (tf_withdrawP[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jkSetup.add (temp);
    // END OF JK PANEL



    /******************** Build "Turret Gun" Tab Panel ********************/

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Look For Target") );
        temp.add ( (useThisInstinct[25] = new JCheckBox() ) );
            useThisInstinct[25].setText ("Use?");
            useThisInstinct[25].setToolTipText ("Use this Instinct?");
            useThisInstinct[25].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 6, 2, 2) );
        for (int i=0 ; i<tf_lookForTarget2P.length ; i++) {
            tf_lookForTarget2P[i] = new JTextField();
            tf_lookForTarget2P[i].setHorizontalAlignment (JTextField.CENTER);
            tf_lookForTarget2P[i].setEnabled (false);
            tf_lookForTarget2P[i].setBackground (Color.lightGray);
            tf_lookForTarget2P[i].setToolTipText (
                resources.getProperty ("LookForTarget" + i));
            subPanel.add (tf_lookForTarget2P[i] );
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        turretSetup.add (temp);


        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Turret Fire") );
        temp.add ( (useThisInstinct[18] = new JCheckBox() ) );
            useThisInstinct[18].setText ("Use?");
            useThisInstinct[18].setToolTipText ("Use This Instinct?");
            useThisInstinct[18].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 5, 2, 2) );
        for (int i=0 ; i<tf_turretFireP.length ; i++) {
            tf_turretFireP[i] = new JTextField();
            tf_turretFireP[i].setHorizontalAlignment (JTextField.CENTER);
            tf_turretFireP[i].setEnabled (false);
            tf_turretFireP[i].setBackground (Color.lightGray);
            tf_turretFireP[i].setToolTipText (
                resources.getProperty ("TurretFire" + i));
            subPanel.add (tf_turretFireP[i]);
        }
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        turretSetup.add (temp);
    // END OF TURRET GUN PANEL




        /******************** Build "MOTS" Tab Panel ********************/

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Charge") );
        temp.add ( (useThisInstinct[19] = new JCheckBox() ) );
            useThisInstinct[19].setText ("Use?");
            useThisInstinct[19].setToolTipText ("Use this instinct?");
            useThisInstinct[19].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (1, 6, 2, 2) );
        for (int i=0 ; i<tf_chargeP.length ; i++) {
            tf_chargeP[i] = new JTextField();
            tf_chargeP[i].setEnabled (false);
            tf_chargeP[i].setBackground (Color.lightGray);
            tf_chargeP[i].setToolTipText (
                resources.getProperty ("Charge" + i));
            subPanel.add (tf_chargeP[i]);
        }
        temp.add (subPanel);
        motsSetup.add (temp);


        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Leap") );
        temp.add ( (useThisInstinct[20] = new JCheckBox() ) );
            useThisInstinct[20].setText ("Use?");
            useThisInstinct[20].setToolTipText ("Use this instinct?");
            useThisInstinct[20].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (1, 6, 2, 2) );
        for (int i=0 ; i<tf_leapP.length ; i++) {
            tf_leapP[i] = new JTextField();
            tf_leapP[i].setEnabled (false);
            tf_leapP[i].setBackground (Color.lightGray);
            tf_leapP[i].setToolTipText (
                resources.getProperty ("Leap" + i));
            subPanel.add (tf_leapP[i]);
        }
        temp.add (subPanel);
        motsSetup.add (temp);


        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Look For Opposing Target") );
        temp.add ( (useThisInstinct[21] = new JCheckBox() ) );
            useThisInstinct[21].setText ("Use?");
            useThisInstinct[21].setToolTipText ("Use this instinct?");
            useThisInstinct[21].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (1, 6, 2, 2) );
        for (int i=0 ; i<tf_lookForOpposingTargetP.length ; i++) {
            tf_lookForOpposingTargetP[i] = new JTextField();
            tf_lookForOpposingTargetP[i].setEnabled (false);
            tf_lookForOpposingTargetP[i].setBackground (Color.lightGray);
            tf_lookForOpposingTargetP[i].setToolTipText (
                resources.getProperty ("LookForOpposingTarget" + i));
            subPanel.add (tf_lookForOpposingTargetP[i]);
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        motsSetup.add (temp);
    // END MOTS SETUP PANEL




        /******************** Build "JEDI" Tab Panel ********************/

        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Force Powers") );
        temp.add ( (useThisInstinct[22] = new JCheckBox() ) );
            useThisInstinct[22].setText ("Use?");
            useThisInstinct[22].setToolTipText ("Use this instinct?");
            useThisInstinct[22].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 7, 2, 2) );
        for (int i=0 ; i<tf_forcePowersP.length ; i++) {
            tf_forcePowersP[i] = new JTextField();
            tf_forcePowersP[i].setEnabled (false);
            tf_forcePowersP[i].setBackground (Color.lightGray);
            tf_forcePowersP[i].setToolTipText (
                resources.getProperty ("ForcePowers" + i));
            subPanel.add (tf_forcePowersP[i]);
        }
        subPanel.add (new JLabel (" ") );
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jediSpecialSetup.add (temp);


        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Saber Fighting") );
        temp.add ( (useThisInstinct[23] = new JCheckBox() ) );
            useThisInstinct[23].setText ("Use?");
            useThisInstinct[23].setToolTipText ("Use this instinct?");
            useThisInstinct[23].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 7, 2, 2) );
        for (int i=0 ; i<tf_saberFightingP.length ; i++) {
            tf_saberFightingP[i] = new JTextField();
            tf_saberFightingP[i].setEnabled (false);
            tf_saberFightingP[i].setBackground (Color.lightGray);
            tf_saberFightingP[i].setToolTipText (
                resources.getProperty ("SaberFighting" + i));
            subPanel.add (tf_saberFightingP[i]);
        }
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jediSpecialSetup.add (temp);


        temp = new JPanel (new GridLayout (2, 1, 2, 2) );
        temp.setBorder (new TitledBorder (
            new EtchedBorder(), "Special Attack") );
        temp.add ( (useThisInstinct[24] = new JCheckBox() ) );
            useThisInstinct[24].setText ("Use?");
            useThisInstinct[24].setToolTipText ("Use this instinct?");
            useThisInstinct[24].addActionListener (checkBoxSetting);
        subPanel = new JPanel (new GridLayout (2, 5, 2, 2) );
        for (int i=0 ; i<tf_specialAttackP.length ; i++) {
            tf_specialAttackP[i] = new JTextField();
            tf_specialAttackP[i].setEnabled (false);
            tf_specialAttackP[i].setBackground (Color.lightGray);
            tf_specialAttackP[i].setToolTipText (
                resources.getProperty("SpecialAttack" + i));
            subPanel.add (tf_specialAttackP[i]);
        }
        subPanel.add (new JLabel (" ") );
        temp.add (subPanel);
        jediSpecialSetup.add (temp);




    /******************** Build Scrollers For Panels ********************/

        JScrollPane initScrollPane = new JScrollPane (initSetup);
        JScrollPane jkScrollPane = new JScrollPane (jkSetup);
        JScrollPane turretScrollPane = new JScrollPane (turretSetup);
        JScrollPane motsScrollPane = new JScrollPane (motsSetup);
        JScrollPane jediScrollPane = new JScrollPane (jediSpecialSetup);



    /*********** Finally Add Components To A Tabbed Interface ***********/

        tabsInterface.addTab (
            "Set-up",
            null,
            initScrollPane,
            "Initial AI Set-up");

        tabsInterface.addTab (
            "JK",
            null,
            jkScrollPane,
            "JK ONLY Options");

        tabsInterface.addTab (
            "Turret Gun",
            null,
            turretScrollPane,
            "Turret Gun");

        tabsInterface.addTab (
            "MotS",
            null,
            motsScrollPane,
            "MotS Options");

        tabsInterface.addTab (
            "Jedi",
            null,
            jediScrollPane,
            "Jedi / Dark Jedi Characteristics");

          // Disable tabs at start-up, until appropriate options are made...
        for (int i=1 ; i<tabsInterface.getTabCount() ; i++)
            tabsInterface.setEnabledAt (i, false);

        tabbedPanel.add (tabsInterface, BorderLayout.CENTER);


        return tabbedPanel;
    }



//=========================================================


    ////////////////////
    //  INNER-CLASSES //
    ////////////////////


    /**
     * This inner-class listens to all radio-buttons.  When
     * the user selects different options, make sure to update
     * accordingly.
     *
     */
    protected class RadioButtonListener implements ItemListener {

        /**
         * This method determines what features should be made available
         * when switching between game types (JK or MotS).  Useful if the
         * editor has made a choice and then changes their mind; this
         * keeps track of the information change-over.
         *
         */
        public void calculateGameMode() {
            if (getGameType().equals ("JK")) {
                rb_alignment[2].setEnabled (false);
                if ( ( (cb_specialCharacterAI[0].isSelected() ) ||
                       (cb_specialCharacterAI[1].isSelected() ) ||
                       (cb_specialCharacterAI[2].isSelected() ) ||
                       (cb_specialCharacterAI[3].isSelected() ) ||
                       (cb_specialCharacterAI[4].isSelected() ) ||
                       (cb_specialCharacterAI[5].isSelected() )
                   ) ) {
                       // disable all checked instinct on tabs
                       // *except for* the JK tab!
                    for (int i=18 ; i<useThisInstinct.length; i++)
                        useThisInstinct[i].setSelected (false);
                    tabsInterface.setEnabledAt (1, true);
                    tabsInterface.setEnabledAt (3, false);
                }
                else {
                    tabsInterface.setEnabledAt (1, false);
                    tabsInterface.setEnabledAt (3, false);
                }
            }
            else
            if (getGameType().equals ("MotS")) {
                rb_alignment[2].setEnabled (true);
                if ( ( (cb_specialCharacterAI[0].isSelected() ) ||
                       (cb_specialCharacterAI[1].isSelected() ) ||
                       (cb_specialCharacterAI[2].isSelected() ) ||
                       (cb_specialCharacterAI[3].isSelected() ) ||
                       (cb_specialCharacterAI[4].isSelected() ) ||
                       (cb_specialCharacterAI[5].isSelected() )
                   ) ) {
                        // turret gun info deselected...
                    //cb_specialCharacterAI[18].setSelected (false);
                    //cb_specialCharacterAI[25].setSelected (false);
                    tabsInterface.setEnabledAt (1, true);
                    tabsInterface.setEnabledAt (3, true);
                }
                else {
                    tabsInterface.setEnabledAt (1, false);
                    tabsInterface.setEnabledAt (3, false);
                }
            }
        }

        /**
         * Item events are handled by the "itemStateChanged()" method.
         *
         * @param event The ItemEvent object responsible for the event.
         */
        public void itemStateChanged (ItemEvent event) {
            if (event.getItem() == rb_gameType[0]) {
                setGameType ("JK");
            }
            else
            if (event.getItem() == rb_gameType[1]) {
                setGameType ("MotS");
            }

            calculateGameMode();


            if (event.getItem() == rb_alignment[0]) {
                setAlignment ("-1.0");
                    // enable - in case we're still disabled
                    // from "rb_alignment[2]" setting
                rb_gameType[0].setEnabled (true);
            }
            else
            if (event.getItem() == rb_alignment[1]) {
                setAlignment ("0.0");
                    // enable - in case we're still disabled
                    // from "rb_alignment[2]" setting
                rb_gameType[0].setEnabled (true);
            }
            else
            if (event.getItem() == rb_alignment[2]) {
                setAlignment ("1.0");
                    // ALLY AI ALIGNMENT
                rb_gameType[0].setEnabled (false);
            }
            else
            if (event.getItem() == rb_difficulty[0]) {      // EASY
                setDifficultyFileNameExtension (
                    GlobalConstants.F_EXT_EASY);
            }
            else
            if (event.getItem() == rb_difficulty[1]) {      // MEDIUM
                setDifficultyFileNameExtension (
                    GlobalConstants.F_EXT_MED);
            }
            else
            if (event.getItem() == rb_difficulty[2]) {      // HARD
                setDifficultyFileNameExtension (
                    GlobalConstants.F_EXT_HARD);
            }
        }

    }   /*  End Of RadioButton Inner-Class  */


//=========================================================


    /**
     * Ever wonder what happens when you click on a checkbox -
     * well it's all contained here...
     *
     */
    protected class CheckBoxListener implements ActionListener {

            /** How "much" an individual object source's value represents. */
        private int objValue = 0;

        /**
         * This method is where the bulk of the processing "power" lies.
         * When passed a powered numerical "value",
         * eg: <br />
         *  <code> Math.pow (x, y) </code> <br />
         * <i>(where the "value" returned is the value of the first argument,
         * raised to the power of the second argument)</i>
         * <br /><br />
         * ... it caculates what features (tabs) should be made accessible
         * corresponding to the user's choice(s).
         * <br /><br />
         * Once these features have been calculated, it will also de-select
         * any of the other "Use This Instinct?" checkboxes
         * (that refer to previous decisions) but still retain the values.
         * This way, inputted data is not deleted - simply provide a valid
         * "Special Character AI" trait and the checkbox may be selected
         * again, and the original (supplied) values (re-)used.
         *
         * @param value The combined value of features selected to use as a
         *  base for calculating which tabs/checkboxes should be
         *  enabled/selected.
         *
         */
        public void calculateFeaturesAvailable (int value) {
            switch (value) {
					// This is if *NOTHING* is currently selected
					// --> enable everything!
                case 0 : {
                    for (int i=1 ; i<tabsInterface.getTabCount() ; i++)
                        tabsInterface.setEnabledAt (i, false);
                    for (int i=0 ; i<useThisInstinct.length ; i++)
                        useThisInstinct[i].setSelected (false);
                    for (int i=0 ; i<cb_specialCharacterAI.length ; i++)
                        cb_specialCharacterAI[i].setEnabled (true);
                    break;
                }
                	// Normal character
                case 1: {
                    for (int i=0 ; i<cb_specialCharacterAI.length ; i++) {
                        if (i == 0)
                            continue;
                        else
                            cb_specialCharacterAI[i].setEnabled (false);
                    }
                    useThisInstinct[18].setSelected (false);     // Disable TurretFire
                    useThisInstinct[25].setSelected (false);     // Disable Turret LookForTarget
                    useThisInstinct[22].setSelected (false);     // Disable ForcePowers
                    useThisInstinct[23].setSelected (false);     // Disable SaberFighting
                    useThisInstinct[24].setSelected (false);     // Disable SpecialAttack
                    tabsInterface.setEnabledAt (2, false);  // Disable TurretGun tab completely...
                    tabsInterface.setEnabledAt (4, false);  // Disable JEDI tab completely...
                    if (getGameType().equals ("JK"))
                        tabsInterface.setEnabledAt (1, true);
                    else
                    if (getGameType().equals ("MotS")) {
						tabsInterface.setEnabledAt (1, true);
                        tabsInterface.setEnabledAt (3, true);
					}
                    break;
                }
                    // Turret Gun
                case 2: {
                    for (int i=0 ; i<cb_specialCharacterAI.length ; i++) {
                        if (i == 1)
                            continue;
                        else
                            cb_specialCharacterAI[i].setEnabled (false);
                    }
                    for (int i=0 ; i<useThisInstinct.length ; i++) {
                        if (i == 18 || i == 25)
                            continue;
                        else
                            useThisInstinct[i].setSelected (false);
                    }
                    tabsInterface.setEnabledAt (2, true);
                    break;
                }
                    // Flying AI - *may* also be used by Jedi!
                    // (which is calculated as switch #: 12 --> 4+8).
                    // On its own though, disable the "Jedi" tab.
				case 8: {
                    for (int i=0 ; i<cb_specialCharacterAI.length ; i++) {
                        if (i == 2 || i == 3)
                            continue;
                        else
                            cb_specialCharacterAI[i].setEnabled (false);
                    }
                        // Deselect TurretGun options if selected...
                    useThisInstinct[18].setSelected (false);
                    useThisInstinct[25].setSelected (false);
                    if (getGameType().equals ("JK"))
                        tabsInterface.setEnabledAt (1, true);
                    else
                    if (getGameType().equals ("MotS")) {
                        tabsInterface.setEnabledAt (1, true);
                        tabsInterface.setEnabledAt (3, true);
                    }
                    tabsInterface.setEnabledAt (4, false);
                    break;
                }
                    // Jedi / Dark Jedi
                case 4:     // fall-through...
				case 12: {
                    for (int i=0 ; i<cb_specialCharacterAI.length ; i++) {
                        if (i == 2 || i == 3)
                            continue;
                        else
                            cb_specialCharacterAI[i].setEnabled (false);
                    }
                    useThisInstinct[18].setSelected (false);
                    useThisInstinct[25].setSelected (false);
					if (getGameType().equals ("JK"))
						tabsInterface.setEnabledAt (1, true);
					else
					if (getGameType().equals ("MotS")) {
						tabsInterface.setEnabledAt (1, true);
						tabsInterface.setEnabledAt (3, true);
					}
					tabsInterface.setEnabledAt (4, true);
					break;
				}
                    // Droid AI
                case 16: {
                    for (int i=0 ; i<cb_specialCharacterAI.length ; i++) {
                        if (i == 4)
                            continue;
                        else
                            cb_specialCharacterAI[i].setEnabled (false);
                    }
                    useThisInstinct[18].setSelected (false);
                    useThisInstinct[25].setSelected (false);
                    useThisInstinct[22].setSelected (false);
                    useThisInstinct[23].setSelected (false);
                    useThisInstinct[24].setSelected (false);
                    tabsInterface.setEnabledAt (1, true);
                    break;
                }
                    // Underwater/swimming AI
                case 32: {
                    for (int i=0 ; i<cb_specialCharacterAI.length ; i++) {
                        if (i == 5)
                            continue;
                        else
                            cb_specialCharacterAI[i].setEnabled (false);
                    }
                    useThisInstinct[18].setSelected (false);
                    useThisInstinct[25].setSelected (false);
                    useThisInstinct[22].setSelected (false);
                    useThisInstinct[23].setSelected (false);
                    useThisInstinct[24].setSelected (false);
                    tabsInterface.setEnabledAt (1, true);
                    break;
                }
            }
        }


        /**
         * Invoked when an action occurs.
         *
         * @param event The event responsible for the action.
         *
         */
        public void actionPerformed (ActionEvent event) {
                // Cast to a generic low-level Object Class data-type
            Object source = event.getSource();

            for (int i=0 ; i<cb_specialCharacterAI.length ; i++) {
                if (source == cb_specialCharacterAI[i]) {
                    if (cb_specialCharacterAI[i].isSelected()) {
						objValue = objValue + (int) Math.pow (2, i);
                    }
                    else {
                        objValue = objValue - (int) Math.pow (2, i);
                    }
                }
            }
            if (GlobalConstants.VERBOSE_DEBUG)
                JOptionPane.showMessageDialog (null,
                    "objValue Ref: " + objValue,
                    "DEBUG MESSAGE",
                JOptionPane.INFORMATION_MESSAGE);
            //System.out.println ("objValue Ref: " + objValue);

            calculateFeaturesAvailable (objValue);


            /* Checkboxes on the JK ONLY Tab Panel */

            if (source == useThisInstinct[0]) {
                if (useThisInstinct[0].isSelected() ) {
                    for (int i=0 ; i<tf_blindFireP.length ; i++) {
                        tf_blindFireP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_blindFireP[i].setEnabled (true);
                        tf_blindFireP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandBlindFire"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_blindFireP.length ; i++) {
                        tf_blindFireP[i].setEnabled (false);
                        tf_blindFireP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[1]) {
                if (useThisInstinct[1].isSelected() ) {
                    for (int i=0 ; i<tf_circleStrafeP.length ; i++) {
                        tf_circleStrafeP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_circleStrafeP[i].setEnabled (true);
                        tf_circleStrafeP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandCircleStrafe"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_circleStrafeP.length ; i++) {
                        tf_circleStrafeP[i].setEnabled (false);
                        tf_circleStrafeP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[2]) {
                if (useThisInstinct[2].isSelected() ) {
                    for (int i=0 ; i<tf_crouchP.length ; i++) {
                        tf_crouchP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_crouchP[i].setEnabled (true);
                        tf_crouchP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandCrouch"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_crouchP.length ; i++) {
                        tf_crouchP[i].setEnabled (false);
                        tf_crouchP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[3]) {
                if (useThisInstinct[3].isSelected() ) {
                    for (int i=0 ; i<tf_dodgeP.length ; i++) {
                        tf_dodgeP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_dodgeP[i].setEnabled (true);
                        tf_dodgeP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandDodge"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_dodgeP.length ; i++) {
                        tf_dodgeP[i].setEnabled (false);
                        tf_dodgeP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[4]) {
                if (useThisInstinct[4].isSelected() ) {
                    for (int i=0 ; i<tf_fleeP.length ; i++) {
                        tf_fleeP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_fleeP[i].setEnabled (true);
                        tf_fleeP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandFlee"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_fleeP.length ; i++) {
                        tf_fleeP[i].setEnabled (false);
                        tf_fleeP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[5]) {
                if (useThisInstinct[5].isSelected() ) {
                    for (int i=0 ; i<tf_followP.length ; i++) {
                        tf_followP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_followP[i].setEnabled (true);
                        tf_followP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandFollow"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_followP.length ; i++) {
                        tf_followP[i].setEnabled (false);
                        tf_followP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[6]) {
                if (useThisInstinct[6].isSelected() ) {
                    for (int i=0 ; i<tf_hitAndRunP.length ; i++) {
                        tf_hitAndRunP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_hitAndRunP[i].setEnabled (true);
                        tf_hitAndRunP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandHitAndRun"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_hitAndRunP.length ; i++) {
                        tf_hitAndRunP[i].setEnabled (false);
                        tf_hitAndRunP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[7]) {
                if (useThisInstinct[7].isSelected() ) {
                    for (int i=0 ; i<tf_jumpP.length ; i++) {
                        tf_jumpP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_jumpP[i].setEnabled (true);
                        tf_jumpP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandJump"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_jumpP.length ; i++) {
                        tf_jumpP[i].setEnabled (false);
                        tf_jumpP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[8]) {
                if (useThisInstinct[8].isSelected() ) {
                    for (int i=0 ; i<tf_listenP.length ; i++) {
                        tf_listenP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_listenP[i].setEnabled (true);
                        tf_listenP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandListen"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_listenP.length ; i++) {
                        tf_listenP[i].setEnabled (false);
                        tf_listenP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[9]) {
                if (useThisInstinct[9].isSelected() ) {
                    for (int i=0 ; i<tf_lobFireP.length ; i++) {
                        tf_lobFireP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_lobFireP[i].setEnabled (true);
                        tf_lobFireP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandLobFire"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_lobFireP.length ; i++) {
                        tf_lobFireP[i].setEnabled (false);
                        tf_lobFireP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[10]) {
                if (useThisInstinct[10].isSelected() ) {
                    for (int i=0 ; i<tf_lookForTargetP.length ; i++) {
                        tf_lookForTargetP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_lookForTargetP[i].setEnabled (true);
                        tf_lookForTargetP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandLookForTarget"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_lookForTargetP.length ; i++) {
                        tf_lookForTargetP[i].setEnabled (false);
                        tf_lookForTargetP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == cb_openDoorsP) {
                if (cb_openDoorsP.isSelected() ) {
                    expandedInfo.setText (
                        resources.getProperty ("expandOpenDoors"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[11]) {
                if (useThisInstinct[11].isSelected() ) {
                    for (int i=0 ; i<tf_primaryFireP.length ; i++) {
                        tf_primaryFireP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_primaryFireP[i].setEnabled (true);
                        tf_primaryFireP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandPrimaryFire"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_primaryFireP.length ; i++) {
                        tf_primaryFireP[i].setEnabled (false);
                        tf_primaryFireP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            if (source == useThisInstinct[12]) {
                if (useThisInstinct[12].isSelected() ) {
                    for (int i=0 ; i<tf_randomTurnP.length ; i++) {
                        tf_randomTurnP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_randomTurnP[i].setEnabled (true);
                        tf_randomTurnP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandRandomTurn"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_randomTurnP.length ; i++) {
                        tf_randomTurnP[i].setEnabled (false);
                        tf_randomTurnP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[13]) {
                if (useThisInstinct[13].isSelected() ) {
                    for (int i=0 ; i<tf_retreatP.length ; i++) {
                        tf_retreatP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_retreatP[i].setEnabled (true);
                        tf_retreatP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandRetreat"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_retreatP.length ; i++) {
                        tf_retreatP[i].setEnabled (false);
                        tf_retreatP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == cb_returnHomeP) {
                if (cb_returnHomeP.isSelected() ) {
                    expandedInfo.setText (
                        resources.getProperty ("expandReturnHome"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[14]) {
                if (useThisInstinct[14].isSelected() ) {
                    for (int i=0 ; i<tf_roamP.length ; i++) {
                        tf_roamP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_roamP[i].setEnabled (true);
                        tf_roamP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandRoam"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_roamP.length ; i++) {
                        tf_roamP[i].setEnabled (false);
                        tf_roamP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[15]) {
                if (useThisInstinct[15].isSelected() ) {
                    for (int i=0 ; i<tf_senseDangerP.length ; i++) {
                        tf_senseDangerP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_senseDangerP[i].setEnabled (true);
                        tf_senseDangerP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandSenseDanger"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_senseDangerP.length ; i++) {
                        tf_senseDangerP[i].setEnabled (false);
                        tf_senseDangerP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[16]) {
                if (useThisInstinct[16].isSelected() ) {
                    for (int i=0 ; i<tf_talkP.length ; i++) {
                        tf_talkP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_talkP[i].setEnabled (true);
                        tf_talkP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandTalk"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_talkP.length ; i++) {
                        tf_talkP[i].setEnabled (false);
                        tf_talkP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[17]) {
                if (useThisInstinct[17].isSelected() ) {
                    for (int i=0 ; i<tf_withdrawP.length ; i++) {
                        tf_withdrawP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_withdrawP[i].setEnabled (true);
                        tf_withdrawP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandWithdraw"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_withdrawP.length ; i++) {
                        tf_withdrawP[i].setEnabled (false);
                        tf_withdrawP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }


                // TURRET GUN PANEL
            else
            if (source == useThisInstinct[25]) {
                if (useThisInstinct[25].isSelected() ) {
                    for (int i=0 ; i<tf_lookForTarget2P.length ; i++) {
                        tf_lookForTarget2P[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_lookForTarget2P[i].setEnabled (true);
                        tf_lookForTarget2P[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandLookForTarget"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_lookForTarget2P.length ; i++) {
                        tf_lookForTarget2P[i].setEnabled (false);
                        tf_lookForTarget2P[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }

            else

            if (source == useThisInstinct[18]) {
                if (useThisInstinct[18].isSelected() ) {
                    for (int i=0 ; i<tf_turretFireP.length ; i++) {
                        tf_turretFireP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_turretFireP[i].setEnabled (true);
                        tf_turretFireP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandTurretFire"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_turretFireP.length ; i++) {
                        tf_turretFireP[i].setEnabled (false);
                        tf_turretFireP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }

                // MOTS PANEL
            else
            if (source == useThisInstinct[19]) {
                if (useThisInstinct[19].isSelected() ) {
                    for (int i=0 ; i<tf_chargeP.length ; i++) {
                        tf_chargeP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_chargeP[i].setEnabled (true);
                        tf_chargeP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty("expandCharge"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_chargeP.length ; i++) {
                        tf_chargeP[i].setEnabled (false);
                        tf_chargeP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[20]) {
                if (useThisInstinct[20].isSelected() ) {
                    for (int i=0 ; i<tf_leapP.length ; i++) {
                        tf_leapP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_leapP[i].setEnabled (true);
                        tf_leapP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandLeap"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_leapP.length ; i++) {
                        tf_leapP[i].setEnabled (false);
                        tf_leapP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[21]) {
                if (useThisInstinct[21].isSelected() ) {
                    for (int i=0 ; i<tf_lookForOpposingTargetP.length ; i++) {
                        tf_lookForOpposingTargetP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_lookForOpposingTargetP[i].setEnabled (true);
                        tf_lookForOpposingTargetP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandLookForOpposingTarget"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_lookForOpposingTargetP.length ; i++) {
                        tf_lookForOpposingTargetP[i].setEnabled (false);
                        tf_lookForOpposingTargetP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }


                // JEDI SETUP PANEL
            else
            if (source == useThisInstinct[22]) {
                if (useThisInstinct[22].isSelected() ) {
                    for (int i=0 ; i<tf_forcePowersP.length ; i++) {
                        tf_forcePowersP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_forcePowersP[i].setEnabled (true);
                        tf_forcePowersP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandForcePowers"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_forcePowersP.length ; i++) {
                        tf_forcePowersP[i].setEnabled (false);
                        tf_forcePowersP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[23]) {
                if (useThisInstinct[23].isSelected() ) {
                    for (int i=0 ; i<tf_saberFightingP.length/2+1 ; i++) {
                        tf_saberFightingP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_saberFightingP[i].setEnabled (true);
                        tf_saberFightingP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandSaberFighting"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_forcePowersP.length ; i++) {
                        tf_saberFightingP[i].setEnabled (false);
                        tf_saberFightingP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
            else
            if (source == useThisInstinct[24]) {
                if (useThisInstinct[24].isSelected() ) {
                    for (int i=0 ; i<tf_specialAttackP.length ; i++) {
                        tf_specialAttackP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_specialAttackP[i].setEnabled (true);
                        tf_specialAttackP[i].setBackground (Color.white);
                    }
                    expandedInfo.setText (
                        resources.getProperty ("expandSpecialAttack"));
                    expandedInfo.setCaretPosition (0);
                }
                else {
                    for (int i=0 ; i<tf_specialAttackP.length ; i++) {
                        tf_specialAttackP[i].setHorizontalAlignment (JTextField.CENTER);
                        tf_specialAttackP[i].setEnabled (false);
                        tf_specialAttackP[i].setBackground (Color.lightGray);
                    }
                    expandedInfo.setText ("Expanded Info");
                }
            }
        }

    }   /*  End Of "CheckBoxListener" Inner-Class   */


//=========================================================


    /**
     * This Inner-Class handles all button-click processes.
     *
     */
    protected class ButtonProcessor implements ActionListener {

            /** Temporary boolean value that monitors whether a
                String has had its "instinct" appended to the front
                of it or not.  If it hasn't, the String will add the
                instinct and reset the flag; if it has (and the flag
                is therefore "true") then the String will append the
                instinct's "x" [number of] arguments to it. */
        boolean done = false;

        /**
         * Invoked when an action occurs.
         *
         * @param event The ActionEvent source that invoked this action.
         *
         */
        public void actionPerformed (ActionEvent event) {

            Object source = event.getSource();

            if (source == generateFile) {
                // Handle "Setup information" error-checking,
                // before processing any further...

                    //		Missing game type:
                    // AI Builder needs to know whether to allow just JK or
                    // MotS (and therefore additionally JK) options...
                if ( ((! rb_gameType[0].isSelected()) &&
                	(! rb_gameType[1].isSelected())) ) {

                    JOptionPane.showMessageDialog (null,
                        resources.getProperty ("errorDesc0"),
                        resources.getProperty ("errorTitle0"),
						JOptionPane.WARNING_MESSAGE);
                    return;
                }

                    //      Missing AI alignment:
                    // AI Builder needs to know the intended alignment for the
                    // AI.  JK uses it just to distinguish between neutral/bad
                    // characters, whereas MotS has a specific "system" for
                    // calculating allies/neutral/nemesis and how to behave
                    // to each type
                if ( ( (! rb_alignment[0].isSelected()) &&
                       (! rb_alignment[1].isSelected() ) &&
                       (! rb_alignment[2].isSelected() )
                   ) ) {
                    JOptionPane.showMessageDialog (null,
                        resources.getProperty ("errorDesc1"),
                        resources.getProperty ("errorTitle1"),
                        JOptionPane.WARNING_MESSAGE);
                    return;
                }

                    //		Incomplete miscellaneous traits:
                    // AI Builder needs these to generate a complete AI File,
                    // so that the AI has a "base" for its perception of its
                    // surroundings, environment and events
                    // (FOV, hearing/sight distances etc).
                if ( tf_miscTraits[1].getText().equals ("") ||
                     tf_miscTraits[2].getText().equals ("") ||
                     tf_miscTraits[3].getText().equals ("") ||
                     tf_miscTraits[4].getText().equals ("") ||
                     tf_miscTraits[5].getText().equals ("")
                   ) {
                    JOptionPane.showMessageDialog (null,
                        resources.getProperty ("errorDesc2") +
                        "\r\n" +
                        "Please view the " + GlobalConstants.DOCUMENTATION +
                        " document for more information regarding the " +
                        "\"AI Setup line\" structure.",
                        resources.getProperty ("errorTitle2"),
                        JOptionPane.WARNING_MESSAGE);
                    return;
                }

                // perform data sanitization --> numbers only!!!
                // --> (to do) here... !!!
                for (int i=2 ; i<tf_miscTraits.length ; i++) {
                    try {
                            // bit of a rigmarole (!!!), but convert the data
                            // to a float value and then reset the text
                            // field's contents with new "parsed" data
                            // (before it's taken in for processing, later
                            // in the method...)
                        float tempValue =
                            Float.parseFloat (tf_miscTraits[i].getText());
                        String realValue = ((Object) tempValue).toString();
                        tf_miscTraits[i].setText (realValue);
                    }
                    catch (NumberFormatException NFE) {
                        JOptionPane.showMessageDialog (null,
                            "The current entry for \"Misc. AI Trait\" " +
                            (i + 1) + " " +
                            "contains an invalid data-type." +
                            "\n" +
                            "This could be due to encountering a string " +
                            "of characters, rather than a number.  " +
                            "Please insert a numerical value. " +
                            "\n" +
                            "Please view the " +
                            GlobalConstants.DOCUMENTATION +
                            " document for more information regarding the " +
                            "\"AI Setup line\" structure.",
                            "Setup Contains Incorrect Datatype",
                        JOptionPane.WARNING_MESSAGE);
                    return;
                    }
                }

                // By this point we should have achieved valid input data.
                // Now just check the "range" of some of the fields, to
                // obtain "correct" data...

                    //      FOV ("Field Of View") Range
                    // Check for valid range - must be between 0 and 360.
                    // If less than the minimum value, return;
                    // If greater than the maximum value, return.
                if (Integer.parseInt (tf_miscTraits[1].getText()) > 360 ||
                    Integer.parseInt (tf_miscTraits[1].getText()) < 0 ) {
                    JOptionPane.showMessageDialog (null,
                        resources.getProperty ("errorDesc4") +
                        "\n" +
                        "Please view the " + GlobalConstants.DOCUMENTATION +
                        " document for more information regarding the " +
                        "\"AI Setup line\" structure.",
                        resources.getProperty ("errorTitle4"),
                        JOptionPane.WARNING_MESSAGE);
                    return;
                }

                    //      Accuracy (% controlling over-all AI accuracy) Range
                    // Check for valid range - must be between 0.0 and 1.0
                    // Exit processing and return if condition fails.
                if (Float.parseFloat (tf_miscTraits[5].getText() ) > 1.0 ||
                    Float.parseFloat (tf_miscTraits[5].getText() ) < 0.0 ) {
                    JOptionPane.showMessageDialog (null,
                        resources.getProperty ("errorDesc5") +
                        "\n" +
                        "Please view the " + GlobalConstants.DOCUMENTATION +
                        " document for more information regarding the " +
                        "\"AI Setup line\" structure.",
                        resources.getProperty ("errorTitle5"),
                        JOptionPane.WARNING_MESSAGE);
                    return;
                }

                    //		No special character AI traits have been chosen:
                    // AI Builder won't know which tabs to enable/disable.
                    // This could result in invalid AI files being generated!
                if ( ( (! cb_specialCharacterAI[0].isSelected() ) &&
                       (! cb_specialCharacterAI[1].isSelected() ) &&
                       (! cb_specialCharacterAI[2].isSelected() ) &&
                       (! cb_specialCharacterAI[3].isSelected() ) &&
                       (! cb_specialCharacterAI[4].isSelected() ) &&
                       (! cb_specialCharacterAI[5].isSelected() )
                   ) ) {
                    JOptionPane.showMessageDialog (null,
                        resources.getProperty ("errorDesc6"),
                        resources.getProperty ("errorTitle6"),
                        JOptionPane.WARNING_MESSAGE);
                    return;
                }


                //################################


                // Start generating the file now...

                codeWindow.setText (getFileHeader() );

                String AIConcatString = "";
                String tempString = "";


                if (useThisInstinct[0].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_blindFireP.length ; i++) {
                        if (tf_blindFireP[i].getText().equals ("")) {
                            JOptionPane.showMessageDialog (null,
                                "BlindFire does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_blindFireP[i].setBackground (Color.red);
                            tf_blindFireP[i].setForeground (Color.yellow);
                            tf_blindFireP[i].requestFocusInWindow();
                            tf_blindFireP[i].selectAll();
                            return;
                        }

                        try {
                            float tempValue =
                                Float.parseFloat (tf_blindFireP[i].getText() );

                            if (GlobalConstants.VERBOSE_DEBUG) {
                                JOptionPane.showMessageDialog (null,
                                "Loop Iteration: " + i + "\n\n" +
                                "Textfield: " + (i + 1) + "\n\n" +
                                "tempValue: " + tempValue + ".",
                                "w00tage",
                                JOptionPane.WARNING_MESSAGE);
                            }

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "BlindFire's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_blindFireP[i].setBackground (Color.red);
                                tf_blindFireP[i].setForeground (Color.yellow);
                                tf_blindFireP[i].requestFocusInWindow();
                                tf_blindFireP[i].selectAll();
                                return;
                            }

                            if (i==1 || i==5) {
                                if (tempValue < 0.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "BlindFire's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_blindFireP[i].setBackground (
                                        Color.red);
                                    tf_blindFireP[i].setForeground (
                                        Color.yellow);
                                    tf_blindFireP[i].requestFocusInWindow();
                                    tf_blindFireP[i].selectAll();
                                    return;
                                }
                            }
                            else
                            if (i == 2) {
                                if (! (tempValue == 0.0 || tempValue == 1.0)) {
                                    JOptionPane.showMessageDialog (null,
                                        "BlindFire's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorBoolean"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_blindFireP[i].setBackground (
                                        Color.red);
                                    tf_blindFireP[i].setForeground (
                                        Color.yellow);
                                    tf_blindFireP[i].requestFocusInWindow();
                                    tf_blindFireP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_blindFireP[i].setBackground (Color.white);
                            tf_blindFireP[i].setForeground (Color.black);
                            tf_blindFireP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for BlindFire.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_blindFireP[i].setBackground (Color.red);
                            tf_blindFireP[i].setForeground (Color.yellow);
                            tf_blindFireP[i].requestFocusInWindow();
                            tf_blindFireP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "BlindFire \t";
                            done = true;
                            tempString += tf_blindFireP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_blindFireP[i].getText() + ", ";
                    }
                    if (done) {
							// Eliminate the last space and comma...
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[1].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_circleStrafeP.length ; i++) {
                        if (tf_circleStrafeP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "CircleStrafe does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
								JOptionPane.WARNING_MESSAGE);
                            tf_circleStrafeP[i].setBackground (
                                Color.red);
                            tf_circleStrafeP[i].setForeground (
                                Color.yellow);
                            tf_circleStrafeP[i].requestFocusInWindow();
                            tf_circleStrafeP[i].selectAll();
                            return;
                        }

                        try {
                            float tempValue =
                                Float.parseFloat (tf_circleStrafeP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "CircleStrafe's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_circleStrafeP[i].setBackground (Color.red);
                                tf_circleStrafeP[i].setForeground (Color.yellow);
                                tf_circleStrafeP[i].requestFocusInWindow();
                                tf_circleStrafeP[i].selectAll();
                                return;
                            }

                            if (i == 4) {
                                if (! (tempValue == 0.0 || tempValue == 1.0)) {
                                    JOptionPane.showMessageDialog (null,
                                        "CircleStrafe's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorBoolean"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_circleStrafeP[i].setBackground (
                                        Color.red);
                                    tf_circleStrafeP[i].setForeground (
                                        Color.yellow);
                                    tf_circleStrafeP[i].requestFocusInWindow();
                                    tf_circleStrafeP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_circleStrafeP[i].setBackground (Color.white);
                            tf_circleStrafeP[i].setForeground (Color.black);
                            tf_circleStrafeP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for CircleStrafe.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_circleStrafeP[i].setBackground (Color.red);
                            tf_circleStrafeP[i].setForeground (Color.yellow);
                            tf_circleStrafeP[i].requestFocusInWindow();
                            tf_circleStrafeP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "CircleStrafe \t";
                            done = true;
                            tempString += tf_circleStrafeP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_circleStrafeP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[2].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_crouchP.length ; i++) {
                        if (tf_crouchP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Crouch does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_crouchP[i].setBackground (Color.red);
                            tf_crouchP[i].setForeground (Color.yellow);
                            tf_crouchP[i].requestFocusInWindow();
                            tf_crouchP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_crouchP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Crouch's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_crouchP[i].setBackground (Color.red);
                                tf_crouchP[i].setForeground (Color.yellow);
                                tf_crouchP[i].requestFocusInWindow();
                                tf_crouchP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_crouchP[i].setBackground (Color.white);
                            tf_crouchP[i].setForeground (Color.black);
                            tf_crouchP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Crouch.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_crouchP[i].setBackground (Color.red);
                            tf_crouchP[i].setForeground (Color.yellow);
                            tf_crouchP[i].requestFocusInWindow();
                            tf_crouchP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Crouch \t";
                            done = true;
                            tempString += tf_crouchP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_crouchP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[3].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_dodgeP.length ; i++) {
                        if (tf_dodgeP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Dodge does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_dodgeP[i].setBackground (Color.red);
                            tf_dodgeP[i].setForeground (Color.yellow);
                            tf_dodgeP[i].requestFocusInWindow();
                            tf_dodgeP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_dodgeP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Dodge's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_dodgeP[i].setBackground (Color.red);
                                tf_dodgeP[i].setForeground (Color.yellow);
                                tf_dodgeP[i].requestFocusInWindow();
                                tf_dodgeP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_dodgeP[i].setBackground (Color.white);
                            tf_dodgeP[i].setForeground (Color.black);
                            tf_dodgeP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Dodge.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_dodgeP[i].setBackground (Color.red);
                            tf_dodgeP[i].setForeground (Color.yellow);
                            tf_dodgeP[i].requestFocusInWindow();
                            tf_dodgeP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Dodge \t";
                            done = true;
                            tempString += tf_dodgeP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_dodgeP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[4].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_fleeP.length ; i++) {
                        if (tf_fleeP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Flee does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_fleeP[i].setBackground (Color.red);
                            tf_fleeP[i].setForeground (Color.yellow);
                            tf_fleeP[i].requestFocusInWindow();
                            tf_fleeP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_fleeP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Flee's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_fleeP[i].setBackground (Color.red);
                                tf_fleeP[i].setForeground (Color.yellow);
                                tf_fleeP[i].requestFocusInWindow();
                                tf_fleeP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_fleeP[i].setBackground (Color.white);
                            tf_fleeP[i].setForeground (Color.black);
                            tf_fleeP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Flee.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_fleeP[i].setBackground (Color.red);
                            tf_fleeP[i].setForeground (Color.yellow);
                            tf_fleeP[i].requestFocusInWindow();
                            tf_fleeP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Flee \t";
                            done = true;
                            tempString += tf_fleeP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_fleeP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }

    //
    //
    // Leave parsing follow until later !!!
    //
    //
                if (useThisInstinct[5].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_followP.length ; i++) {
                        if (tf_followP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Follow does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            return;
                        }

                        if (!done) {
                            tempString = "Follow \t";
                            done = true;
                            tempString += tf_followP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_followP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[6].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_hitAndRunP.length ; i++) {
                        if (tf_hitAndRunP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "HitAndRun does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_hitAndRunP[i].setBackground (Color.red);
                            tf_hitAndRunP[i].setForeground (Color.yellow);
                            tf_hitAndRunP[i].requestFocusInWindow();
                            tf_hitAndRunP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_hitAndRunP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "HitAndRun's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_hitAndRunP[i].setBackground (Color.red);
                                tf_hitAndRunP[i].setForeground (Color.yellow);
                                tf_hitAndRunP[i].requestFocusInWindow();
                                tf_hitAndRunP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_hitAndRunP[i].setBackground (Color.white);
                            tf_hitAndRunP[i].setForeground (Color.black);
                            tf_hitAndRunP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for HitAndRun.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_hitAndRunP[i].setBackground (Color.red);
                            tf_hitAndRunP[i].setForeground (Color.yellow);
                            tf_hitAndRunP[i].requestFocusInWindow();
                            tf_hitAndRunP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "HitAndRun \t";
                            done = true;
                            tempString += tf_hitAndRunP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_hitAndRunP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[7].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_jumpP.length ; i++) {
                        if (tf_jumpP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Jump does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_jumpP[i].setBackground (Color.red);
                            tf_jumpP[i].setForeground (Color.yellow);
                            tf_jumpP[i].requestFocusInWindow();
                            tf_jumpP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_jumpP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Jump's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_jumpP[i].setBackground (Color.red);
                                tf_jumpP[i].setForeground (Color.yellow);
                                tf_jumpP[i].requestFocusInWindow();
                                tf_jumpP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_jumpP[i].setBackground (Color.white);
                            tf_jumpP[i].setForeground (Color.black);
                            tf_jumpP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Jump.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_jumpP[i].setBackground (Color.red);
                            tf_jumpP[i].setForeground (Color.yellow);
                            tf_jumpP[i].requestFocusInWindow();
                            tf_jumpP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Jump \t";
                            done = true;
                            tempString += tf_jumpP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_jumpP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[8].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_listenP.length ; i++) {
                        if (tf_listenP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Listen does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_listenP[i].setBackground (Color.red);
                            tf_listenP[i].setForeground (Color.yellow);
                            tf_listenP[i].requestFocusInWindow();
                            tf_listenP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_listenP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Listen's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_listenP[i].setBackground (Color.red);
                                tf_listenP[i].setForeground (Color.yellow);
                                tf_listenP[i].requestFocusInWindow();
                                tf_listenP[i].selectAll();
                                return;
                            }

                            if (i == 0) {
                                if (tempValue < 0.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "Listen's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_listenP[i].setBackground (
                                        Color.red);
                                    tf_listenP[i].setForeground (
                                        Color.yellow);
                                    tf_listenP[i].requestFocusInWindow();
                                    tf_listenP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_listenP[i].setBackground (Color.white);
                            tf_listenP[i].setForeground (Color.black);
                            tf_listenP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Listen.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_listenP[i].setBackground (Color.red);
                            tf_listenP[i].setForeground (Color.yellow);
                            tf_listenP[i].requestFocusInWindow();
                            tf_listenP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Listen \t";
                            done = true;
                            tempString += tf_listenP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_listenP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[9].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_lobFireP.length ; i++) {
                        if (tf_lobFireP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "LobFire does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_lobFireP[i].setBackground (Color.red);
                            tf_lobFireP[i].setForeground (Color.yellow);
                            tf_lobFireP[i].requestFocusInWindow();
                            tf_lobFireP[i].selectAll();
                            return;
                        }

                        try {
                            float tempValue =
                                Float.parseFloat (tf_lobFireP[i].getText() );

                            if (i == 0) {
                                if (tempValue < -1.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "LobFire's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_lobFireP[i].setBackground (
                                        Color.red);
                                    tf_lobFireP[i].setForeground (
                                        Color.yellow);
                                    tf_lobFireP[i].requestFocusInWindow();
                                    tf_lobFireP[i].selectAll();
                                    return;
                                }
                            }
                            else {
                                if (tempValue < 0) {
                                    JOptionPane.showMessageDialog (null,
                                        "LobFire's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorNegative"),
                                        resources.getProperty (
                                            "instinctDefTitle3"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_lobFireP[i].setBackground (Color.red);
                                    tf_lobFireP[i].setForeground (Color.yellow);
                                    tf_lobFireP[i].requestFocusInWindow();
                                    tf_lobFireP[i].selectAll();
                                    return;
                                }

                                if (i == 4 || i == 5) {
                                    if (tempValue < 0.0 || tempValue > 1.0) {
                                        JOptionPane.showMessageDialog (null,
                                            "LobFire's argument " +
                                            (i + 1) +
                                            resources.getProperty (
                                                "errorBoolean"),
                                            resources.getProperty (
                                                "instinctDefTitle1"),
                                            JOptionPane.WARNING_MESSAGE);
                                        tf_lobFireP[i].setBackground (
                                            Color.red);
                                        tf_lobFireP[i].setForeground (
                                            Color.yellow);
                                        tf_lobFireP[i].requestFocusInWindow();
                                        tf_lobFireP[i].selectAll();
                                        return;
                                    }
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_lobFireP[i].setBackground (Color.white);
                            tf_lobFireP[i].setForeground (Color.black);
                            tf_lobFireP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for LobFire.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_lobFireP[i].setBackground (Color.red);
                            tf_lobFireP[i].setForeground (Color.yellow);
                            tf_lobFireP[i].requestFocusInWindow();
                            tf_lobFireP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "LobFire \t";
                            done = true;
                            tempString += tf_lobFireP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_lobFireP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[10].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_lookForTargetP.length ; i++) {
                        if (tf_lookForTargetP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "LookForTarget does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_lookForTargetP[i].setBackground (Color.red);
                            tf_lookForTargetP[i].setForeground (Color.yellow);
                            tf_lookForTargetP[i].requestFocusInWindow();
                            tf_lookForTargetP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_lookForTargetP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "LookForTarget's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_lookForTargetP[i].setBackground (Color.red);
                                tf_lookForTargetP[i].setForeground (Color.yellow);
                                tf_lookForTargetP[i].requestFocusInWindow();
                                tf_lookForTargetP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_lookForTargetP[i].setBackground (Color.white);
                            tf_lookForTargetP[i].setForeground (Color.black);
                            tf_lookForTargetP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for LookForTarget.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_lookForTargetP[i].setBackground (Color.red);
                            tf_lookForTargetP[i].setForeground (Color.yellow);
                            tf_lookForTargetP[i].requestFocusInWindow();
                            tf_lookForTargetP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "LookForTarget \t";
                            done = true;
                            tempString += tf_lookForTargetP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_lookForTargetP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (cb_openDoorsP.isSelected() ) {
                    tempString = "OpenDoors" + "\n";
                    AIConcatString += tempString;
                }


                if (useThisInstinct[11].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_primaryFireP.length ; i++) {
                        if (tf_primaryFireP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "PrimaryFire does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_primaryFireP[i].setBackground (Color.red);
                            tf_primaryFireP[i].setForeground (Color.yellow);
                            tf_primaryFireP[i].requestFocusInWindow();
                            tf_primaryFireP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_primaryFireP[i].getText() );

                            if (i == 1) {
                                if (tempValue < -1.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "PrimaryFire's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_primaryFireP[i].setBackground (
                                        Color.red);
                                    tf_primaryFireP[i].setForeground (
                                        Color.yellow);
                                    tf_primaryFireP[i].requestFocusInWindow();
                                    tf_primaryFireP[i].selectAll();
                                    return;
                                }
                            }
                            else {
                                if (tempValue < 0) {
                                    JOptionPane.showMessageDialog (null,
                                        "PrimaryFire's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorNegative"),
                                        resources.getProperty (
                                            "instinctDefTitle3"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_primaryFireP[i].setBackground (Color.red);
                                    tf_primaryFireP[i].setForeground (Color.yellow);
                                    tf_primaryFireP[i].requestFocusInWindow();
                                    tf_primaryFireP[i].selectAll();
                                    return;
                                }

                                if (i == 3 || i == 6 || i == 7) {
                                    if (tempValue < 0.0 || tempValue > 1.0) {
                                        JOptionPane.showMessageDialog (null,
                                            "PrimaryFire's argument " +
                                            (i + 1) +
                                            resources.getProperty ("errorBoolean"),
                                            resources.getProperty ("instinctDefTitle1"),
                                            JOptionPane.WARNING_MESSAGE);
                                        tf_primaryFireP[i].setBackground (Color.red);
                                        tf_primaryFireP[i].setForeground (Color.yellow);
                                        tf_primaryFireP[i].requestFocusInWindow();
                                        tf_primaryFireP[i].selectAll();
                                        return;
                                    }
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_primaryFireP[i].setBackground (Color.white);
                            tf_primaryFireP[i].setForeground (Color.black);
                            tf_primaryFireP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for PrimaryFire.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_primaryFireP[i].setBackground (Color.red);
                            tf_primaryFireP[i].setForeground (Color.yellow);
                            tf_primaryFireP[i].requestFocusInWindow();
                            tf_primaryFireP[i].selectAll();
                            return;
                        }


                        if (!done) {
                            tempString = "PrimaryFire \t";
                            done = true;
                            tempString += tf_primaryFireP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_primaryFireP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[12].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_randomTurnP.length ; i++) {
                        if (tf_randomTurnP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "RandomTurn does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_randomTurnP[i].setBackground (Color.red);
                            tf_randomTurnP[i].setForeground (Color.yellow);
                            tf_randomTurnP[i].requestFocusInWindow();
                            tf_randomTurnP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_randomTurnP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "RandomTurn's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_randomTurnP[i].setBackground (Color.red);
                                tf_randomTurnP[i].setForeground (Color.yellow);
                                tf_randomTurnP[i].requestFocusInWindow();
                                tf_randomTurnP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_randomTurnP[i].setBackground (Color.white);
                            tf_randomTurnP[i].setForeground (Color.black);
                            tf_randomTurnP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for RandomTurn.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_randomTurnP[i].setBackground (Color.red);
                            tf_randomTurnP[i].setForeground (Color.yellow);
                            tf_randomTurnP[i].requestFocusInWindow();
                            tf_randomTurnP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "RandomTurn \t";
                            done = true;
                            tempString += tf_randomTurnP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_randomTurnP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[13].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_retreatP.length ; i++) {
                        if (tf_retreatP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Retreat does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_retreatP[i].setBackground (Color.red);
                            tf_retreatP[i].setForeground (Color.yellow);
                            tf_retreatP[i].requestFocusInWindow();
                            tf_retreatP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_retreatP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Retreat's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_retreatP[i].setBackground (Color.red);
                                tf_retreatP[i].setForeground (Color.yellow);
                                tf_retreatP[i].requestFocusInWindow();
                                tf_retreatP[i].selectAll();
                                return;
                            }

                            if (i == 0 || i == 1) {
                                if(tempValue < 0.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "Retreat's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_retreatP[i].setBackground (Color.red);
                                    tf_retreatP[i].setForeground (Color.yellow);
                                    tf_retreatP[i].requestFocusInWindow();
                                    tf_retreatP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_retreatP[i].setBackground (Color.white);
                            tf_retreatP[i].setForeground (Color.black);
                            tf_retreatP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Retreat.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_retreatP[i].setBackground (Color.red);
                            tf_retreatP[i].setForeground (Color.yellow);
                            tf_retreatP[i].requestFocusInWindow();
                            tf_retreatP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Retreat \t";
                            done = true;
                            tempString += tf_retreatP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_retreatP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (cb_returnHomeP.isSelected() ) {
                    tempString = "ReturnHome" + "\n";
                    AIConcatString += tempString;
                }


                if (useThisInstinct[14].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_roamP.length ; i++) {
                        if (tf_roamP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Roam does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_roamP[i].setBackground (Color.red);
                            tf_roamP[i].setForeground (Color.yellow);
                            tf_roamP[i].requestFocusInWindow();
                            tf_roamP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_roamP[i].getText() );

                            if (i == 0) {
                                if (tempValue < 0) {
                                    JOptionPane.showMessageDialog (null,
                                        "Roam's argument " +
                                        (i + 1) +
                                        resources.getProperty (
                                            "errorNegative"),
                                        resources.getProperty (
                                            "instinctDefTitle3"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_roamP[i].setBackground (Color.red);
                                    tf_roamP[i].setForeground (Color.yellow);
                                    tf_roamP[i].requestFocusInWindow();
                                    tf_roamP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_roamP[i].setBackground (Color.white);
                            tf_roamP[i].setForeground (Color.black);
                            tf_roamP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Roam.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_roamP[i].setBackground (Color.red);
                            tf_roamP[i].setForeground (Color.yellow);
                            tf_roamP[i].requestFocusInWindow();
                            tf_roamP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Roam \t";
                            done = true;
                            tempString += tf_roamP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_roamP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


    //
    // May need to alter "SenseDanger", pending further testing of stats...
    //

                if (useThisInstinct[15].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_senseDangerP.length ; i++) {
                        if (tf_senseDangerP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "SenseDanger does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_senseDangerP[i].setBackground (Color.red);
                            tf_senseDangerP[i].setForeground (Color.yellow);
                            tf_senseDangerP[i].requestFocusInWindow();
                            tf_senseDangerP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_senseDangerP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "SenseDanger's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_senseDangerP[i].setBackground (Color.red);
                                tf_senseDangerP[i].setForeground (Color.yellow);
                                tf_senseDangerP[i].requestFocusInWindow();
                                tf_senseDangerP[i].selectAll();
                                return;
                            }

                            if (i == 0) {
                                if (tempValue > 100.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "SenseDanger's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_senseDangerP[i].setBackground (Color.red);
                                    tf_senseDangerP[i].setForeground (Color.yellow);
                                    tf_senseDangerP[i].requestFocusInWindow();
                                    tf_senseDangerP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_senseDangerP[i].setBackground (Color.white);
                            tf_senseDangerP[i].setForeground (Color.black);
                            tf_senseDangerP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for SenseDanger.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_senseDangerP[i].setBackground (Color.red);
                            tf_senseDangerP[i].setForeground (Color.yellow);
                            tf_senseDangerP[i].requestFocusInWindow();
                            tf_senseDangerP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "SenseDanger \t";
                            done = true;
                            tempString += tf_senseDangerP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_senseDangerP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[16].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_talkP.length ; i++) {
                        if (tf_talkP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Talk does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_talkP[i].setBackground (Color.red);
                            tf_talkP[i].setForeground (Color.yellow);
                            tf_talkP[i].requestFocusInWindow();
                            tf_talkP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_talkP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Talk's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_talkP[i].setBackground (Color.red);
                                tf_talkP[i].setForeground (Color.yellow);
                                tf_talkP[i].requestFocusInWindow();
                                tf_talkP[i].selectAll();
                                return;
                            }

                            if (i == 1) {
                                if (tempValue < 0.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "Talk's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_talkP[i].setBackground (Color.red);
                                    tf_talkP[i].setForeground (Color.yellow);
                                    tf_talkP[i].requestFocusInWindow();
                                    tf_talkP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_talkP[i].setBackground (Color.white);
                            tf_talkP[i].setForeground (Color.black);
                            tf_talkP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Talk.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_talkP[i].setBackground (Color.red);
                            tf_talkP[i].setForeground (Color.yellow);
                            tf_talkP[i].requestFocusInWindow();
                            tf_talkP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Talk \t";
                            done = true;
                            tempString += tf_talkP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_talkP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[17].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_withdrawP.length ; i++) {
                        if (tf_withdrawP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Withdraw does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_withdrawP[i].setBackground (Color.red);
                            tf_withdrawP[i].setForeground (Color.yellow);
                            tf_withdrawP[i].requestFocusInWindow();
                            tf_withdrawP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_withdrawP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Withdraw's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_withdrawP[i].setBackground (Color.red);
                                tf_withdrawP[i].setForeground (Color.yellow);
                                tf_withdrawP[i].requestFocusInWindow();
                                tf_withdrawP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_withdrawP[i].setBackground (Color.white);
                            tf_withdrawP[i].setForeground (Color.black);
                            tf_withdrawP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Withdraw.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_withdrawP[i].setBackground (Color.red);
                            tf_withdrawP[i].setForeground (Color.yellow);
                            tf_withdrawP[i].requestFocusInWindow();
                            tf_withdrawP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Withdraw \t";
                            done = true;
                            tempString += tf_withdrawP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_withdrawP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }

// OUT OF ORDER, DUE TO LATE IMPLEMENTATION
                if (useThisInstinct[25].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_lookForTarget2P.length ; i++) {
                        if (tf_lookForTarget2P[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "LookForTarget does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_lookForTarget2P[i].setBackground (Color.red);
                            tf_lookForTarget2P[i].setForeground (Color.yellow);
                            tf_lookForTarget2P[i].requestFocusInWindow();
                            tf_lookForTarget2P[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_lookForTarget2P[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "LookForTarget's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_lookForTarget2P[i].setBackground (Color.red);
                                tf_lookForTarget2P[i].setForeground (Color.yellow);
                                tf_lookForTarget2P[i].requestFocusInWindow();
                                tf_lookForTarget2P[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_lookForTarget2P[i].setBackground (Color.white);
                            tf_lookForTarget2P[i].setForeground (Color.black);
                            tf_lookForTarget2P[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for LookForTarget.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_lookForTarget2P[i].setBackground (Color.red);
                            tf_lookForTarget2P[i].setForeground (Color.yellow);
                            tf_lookForTarget2P[i].requestFocusInWindow();
                            tf_lookForTarget2P[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "LookForTarget \t";
                            done = true;
                            tempString += tf_lookForTarget2P[i].getText() + ", ";
                        }
                        else
                            tempString += tf_lookForTarget2P[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }
// GENERATE REST OF SEQUENCE...

                if (useThisInstinct[18].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_turretFireP.length ; i++) {
                        if (tf_turretFireP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "TurretFire does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_turretFireP[i].setBackground (Color.red);
                            tf_turretFireP[i].setForeground (Color.yellow);
                            tf_turretFireP[i].requestFocusInWindow();
                            tf_turretFireP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_turretFireP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "TurretFire's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_turretFireP[i].setBackground (Color.red);
                                tf_turretFireP[i].setForeground (Color.yellow);
                                tf_turretFireP[i].requestFocusInWindow();
                                tf_turretFireP[i].selectAll();
                                return;
                            }

                            if (i == 2 || i == 3) {
                                if (tempValue > 180.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "TurretFire's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_turretFireP[i].setBackground (Color.red);
                                    tf_turretFireP[i].setForeground (Color.yellow);
                                    tf_turretFireP[i].requestFocusInWindow();
                                    tf_turretFireP[i].selectAll();
                                    return;
                                }
                            }

                            if (i == 5 || i ==8) {
                                if (tempValue < 0.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "TurretFire's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_turretFireP[i].setBackground (Color.red);
                                    tf_turretFireP[i].setForeground (Color.yellow);
                                    tf_turretFireP[i].requestFocusInWindow();
                                    tf_turretFireP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_turretFireP[i].setBackground (Color.white);
                            tf_turretFireP[i].setForeground (Color.black);
                            tf_turretFireP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for TurretFire.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_turretFireP[i].setBackground (Color.red);
                            tf_turretFireP[i].setForeground (Color.yellow);
                            tf_turretFireP[i].requestFocusInWindow();
                            tf_turretFireP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "TurretFire \t";
                            done = true;
                            tempString += tf_turretFireP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_turretFireP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }

    //
    // Charge requires further testing, before setting argument constraints
    // around possible values.
    //

                if (useThisInstinct[19].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_chargeP.length ; i++) {
                        if (tf_chargeP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Charge does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_chargeP[i].setBackground (Color.red);
                            tf_chargeP[i].setForeground (Color.yellow);
                            tf_chargeP[i].requestFocusInWindow();
                            tf_chargeP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_chargeP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "Charge's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_chargeP[i].setBackground (Color.red);
                                tf_chargeP[i].setForeground (Color.yellow);
                                tf_chargeP[i].requestFocusInWindow();
                                tf_chargeP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_chargeP[i].setBackground (Color.white);
                            tf_chargeP[i].setForeground (Color.black);
                            tf_chargeP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Charge.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_chargeP[i].setBackground (Color.red);
                            tf_chargeP[i].setForeground (Color.yellow);
                            tf_chargeP[i].requestFocusInWindow();
                            tf_chargeP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Charge \t";
                            done = true;
                            tempString += tf_chargeP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_chargeP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[20].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_leapP.length ; i++) {
                        if (tf_leapP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "Leap does not have sufficient argument " +
                                "information to complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_leapP[i].setBackground (Color.red);
                            tf_leapP[i].setForeground (Color.yellow);
                            tf_leapP[i].requestFocusInWindow();
                            tf_leapP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_leapP[i].getText() );


                            if (i == 1) {
                                if (tempValue < -1.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "Leap's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_leapP[i].setBackground (Color.red);
                                    tf_leapP[i].setForeground (Color.yellow);
                                    tf_leapP[i].requestFocusInWindow();
                                    tf_leapP[i].selectAll();
                                    return;
                                }
                            }
                            else {
                                if (tempValue < 0) {
                                    JOptionPane.showMessageDialog (null,
                                        "Leap's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorNegative"),
                                        resources.getProperty (
                                            "instinctDefTitle3"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_leapP[i].setBackground (Color.red);
                                    tf_leapP[i].setForeground (Color.yellow);
                                    tf_leapP[i].requestFocusInWindow();
                                    tf_leapP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_leapP[i].setBackground (Color.white);
                            tf_leapP[i].setForeground (Color.black);
                            tf_leapP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for Leap.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_leapP[i].setBackground (Color.red);
                            tf_leapP[i].setForeground (Color.yellow);
                            tf_leapP[i].requestFocusInWindow();
                            tf_leapP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "Leap \t";
                            done = true;
                            tempString += tf_leapP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_leapP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[21].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_lookForOpposingTargetP.length ; i++) {
                        if (tf_lookForOpposingTargetP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "LookForOpposingTarget does not have " +
                                "sufficient argument information to " +
                                "complete this instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_lookForOpposingTargetP[i].setBackground (Color.red);
                            tf_lookForOpposingTargetP[i].setForeground (Color.yellow);
                            tf_lookForOpposingTargetP[i].requestFocusInWindow();
                            tf_lookForOpposingTargetP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_lookForOpposingTargetP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "SenseDanger's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_lookForOpposingTargetP[i].setBackground (Color.red);
                                tf_lookForOpposingTargetP[i].setForeground (Color.yellow);
                                tf_lookForOpposingTargetP[i].requestFocusInWindow();
                                tf_lookForOpposingTargetP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_lookForOpposingTargetP[i].setBackground (Color.white);
                            tf_lookForOpposingTargetP[i].setForeground (Color.black);
                            tf_lookForOpposingTargetP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for LookForOpposingTarget.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_lookForOpposingTargetP[i].setBackground (Color.red);
                            tf_lookForOpposingTargetP[i].setForeground (Color.yellow);
                            tf_lookForOpposingTargetP[i].requestFocusInWindow();
                            tf_lookForOpposingTargetP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "LookForOpposingTarget \t";
                            done = true;
                            tempString += tf_lookForOpposingTargetP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_lookForOpposingTargetP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[22].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_forcePowersP.length ; i++) {
                        if (tf_forcePowersP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "ForcePowers does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_forcePowersP[i].setBackground (Color.red);
                            tf_forcePowersP[i].setForeground (Color.yellow);
                            tf_forcePowersP[i].requestFocusInWindow();
                            tf_forcePowersP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_forcePowersP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "SenseDanger's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_forcePowersP[i].setBackground (Color.red);
                                tf_forcePowersP[i].setForeground (Color.yellow);
                                tf_forcePowersP[i].requestFocusInWindow();
                                tf_forcePowersP[i].selectAll();
                                return;
                            }

                            if (i == 7 || i == 8 || i == 9) {
                                if (tempValue < 0.0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "SenseDanger's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_forcePowersP[i].setBackground (Color.red);
                                    tf_forcePowersP[i].setForeground (Color.yellow);
                                    tf_forcePowersP[i].requestFocusInWindow();
                                    tf_forcePowersP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_forcePowersP[i].setBackground (Color.white);
                            tf_forcePowersP[i].setForeground (Color.black);
                            tf_forcePowersP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for ForcePowers.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_forcePowersP[i].setBackground (Color.red);
                            tf_forcePowersP[i].setForeground (Color.yellow);
                            tf_forcePowersP[i].requestFocusInWindow();
                            tf_forcePowersP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "ForcePowers \t";
                            done = true;
                            tempString += tf_forcePowersP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_forcePowersP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[23].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_saberFightingP.length/2+1 ; i++) {
                        if (tf_saberFightingP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "SaberFighting does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_saberFightingP[i].setBackground (Color.red);
                            tf_saberFightingP[i].setForeground (Color.yellow);
                            tf_saberFightingP[i].requestFocusInWindow();
                            tf_saberFightingP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_saberFightingP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "SenseDanger's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_saberFightingP[i].setBackground (Color.red);
                                tf_saberFightingP[i].setForeground (Color.yellow);
                                tf_saberFightingP[i].requestFocusInWindow();
                                tf_saberFightingP[i].selectAll();
                                return;
                            }

                            if (i == 0) {
                                if (tempValue < 0 || tempValue > 1.0) {
                                    JOptionPane.showMessageDialog (null,
                                        "SaberFighting's argument " +
                                        (i + 1) +
                                        resources.getProperty ("errorOORV"),
                                        resources.getProperty (
                                            "instinctDefTitle1"),
                                        JOptionPane.WARNING_MESSAGE);
                                    tf_saberFightingP[i].setBackground (Color.red);
                                    tf_saberFightingP[i].setForeground (Color.yellow);
                                    tf_saberFightingP[i].requestFocusInWindow();
                                    tf_saberFightingP[i].selectAll();
                                    return;
                                }
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_saberFightingP[i].setBackground (Color.white);
                            tf_saberFightingP[i].setForeground (Color.black);
                            tf_saberFightingP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for SaberFighting.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_saberFightingP[i].setBackground (Color.red);
                            tf_saberFightingP[i].setForeground (Color.yellow);
                            tf_saberFightingP[i].requestFocusInWindow();
                            tf_saberFightingP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "SaberFighting \t";
                            done = true;
                            tempString += tf_saberFightingP[i].getText() + ", ";
                        }
                        else
                            tempString += tf_saberFightingP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                        	tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }


                if (useThisInstinct[24].isSelected() ) {
                    boolean done = false;
                    for (int i=0 ; i<tf_specialAttackP.length ; i++) {
                        if (tf_specialAttackP[i].getText().equals ("") ) {
                            JOptionPane.showMessageDialog (null,
                                "SpecialAttack does not have sufficient " +
                                "argument information to complete this " +
                                "instinct." +
                                "\n" +
                                "First error encountered: \t argument " +
                                (i + 1) +
                                ".",
                                resources.getProperty ("instinctDefTitle0"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_specialAttackP[i].setBackground (Color.red);
                            tf_specialAttackP[i].setForeground (Color.yellow);
                            tf_specialAttackP[i].requestFocusInWindow();
                            tf_specialAttackP[i].selectAll();
                            return;
                        }


                        try {
                            float tempValue =
                                Float.parseFloat (tf_specialAttackP[i].getText() );

                            if (tempValue < 0) {
                                JOptionPane.showMessageDialog (null,
                                    "SpecialAttack's argument " +
                                    (i + 1) +
                                    resources.getProperty ("errorNegative"),
                                    resources.getProperty (
                                        "instinctDefTitle3"),
                                    JOptionPane.WARNING_MESSAGE);
                                tf_specialAttackP[i].setBackground (Color.red);
                                tf_specialAttackP[i].setForeground (Color.yellow);
                                tf_specialAttackP[i].requestFocusInWindow();
                                tf_specialAttackP[i].selectAll();
                                return;
                            }

                            String realValue = ((Object) tempValue).toString();
                            tf_specialAttackP[i].setBackground (Color.white);
                            tf_specialAttackP[i].setForeground (Color.black);
                            tf_specialAttackP[i].setText (realValue);
                        }
                        catch (NumberFormatException NFE) {
                            JOptionPane.showMessageDialog (null,
                                "A non-numerical value was found for argument " +
                                (i + 1) +
                                " for SpecialAttack.  Please supply a correct value.",
                                resources.getProperty ("instinctDefTitle2"),
                                JOptionPane.WARNING_MESSAGE);
                            tf_specialAttackP[i].setBackground (Color.red);
                            tf_specialAttackP[i].setForeground (Color.yellow);
                            tf_specialAttackP[i].requestFocusInWindow();
                            tf_specialAttackP[i].selectAll();
                            return;
                        }

                        if (!done) {
                            tempString = "SpecialAttack \t";
                            done = true;
                            tempString +=
                                tf_specialAttackP[i].getText() + ", ";
                        }
                        else
                            tempString +=
                                tf_specialAttackP[i].getText() + ", ";
                    }
                    if (done) {
                        tempString =
                            tempString.substring (0, (tempString.length()-2) );
                        tempString += "\n";
                        AIConcatString += tempString;
                    }
                }

                    // Now pull it all together to append to the file header
                    // and AI setup line Strings...
                codeWindow.append (AIConcatString + "\r\n\n");
            }
            else
            if (source == clearCodeWindow) {
                codeWindow.setText ("");    // Simple enough...
            }
            else
            if (source == saveFile) {
                String nameAndExtension = null;
                if (codeWindow.getText().equals ("") ) {
                    JOptionPane.showMessageDialog ( null,
                        resources.getProperty ("errorDesc7"),
                        resources.getProperty ("errorTitle7"),
                        JOptionPane.INFORMATION_MESSAGE );
                }
                else
                if (getDifficultyFileNameExtension().equals ("") ||
                    getGameType().equals ("") ) {

                    JOptionPane.showMessageDialog ( null,
                        resources.getProperty ("errorDesc8"),
                        resources.getProperty ("errorTitle8"),
                    JOptionPane.INFORMATION_MESSAGE );
                    return;
                }
                else {
                    JFileChooser fileChooser = new JFileChooser();
                    ExtensionFilter filter = new ExtensionFilter (
                        getDifficultyFileNameExtension(),
                        "AI File Difficulty (*" +
                        getDifficultyFileNameExtension() + ")" );

                    fileChooser.addChoosableFileFilter (filter);
                    fileChooser.setFileFilter (filter);
                    fileChooser.showSaveDialog (null);

                    File fileName = fileChooser.getSelectedFile();
                    if (fileName == null)   // If "save dialog" is cancelled
                        return;

                    //nameAndExtension =
                   	//getFileName() + getDifficultyFileNameExtension();
                   	// --> Decided against this implementation;
                   	// rather have the line below...
                    nameAndExtension =
                        fileChooser.getSelectedFile().getName() +
                        getDifficultyFileNameExtension();

                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (fileChooser.getCurrentDirectory() +
                        " || " + nameAndExtension);

                    File thisFile = new File (
                        fileChooser.getCurrentDirectory(), nameAndExtension);

                    if (thisFile.exists() ) {
                        int result =
                            JOptionPane.showConfirmDialog (
                                (Component) event.getSource(),
                                "File " + fileName + " already exists." +
                                resources.getProperty ("errorDesc9"),
                                resources.getProperty ("errorTitle9"),
                                JOptionPane.YES_NO_OPTION,
                                JOptionPane.WARNING_MESSAGE);

                        if (result == fileChooser.APPROVE_OPTION) {
                            try {
                                new DataRecordOperations().writeFile (
                                    thisFile, codeWindow.getText());
                            }
                            catch (IOException ioErr) {
                                JOptionPane.showMessageDialog ( null,
                                    resources.getProperty ("errorDesc10"),
                                    resources.getProperty ("errorTitle10"),
                                JOptionPane.ERROR_MESSAGE);
                            }
                        }
                        else
                            return;
                    }
                    else {
                        try {
                            new DataRecordOperations().writeFile (
                                thisFile, codeWindow.getText() );

                            setStatusBarText (
                                "Last saved file: " +
                            	thisFile.getCanonicalPath() );
                        }
                        catch (IOException ioerr) {
                            JOptionPane.showMessageDialog ( null,
                                resources.getProperty ("errorDesc10"),
                                resources.getProperty ("errorTitle10"),
                            JOptionPane.ERROR_MESSAGE);
                        }
                    }
                }
            }
            else
            if (source == openFile) {
                JFileChooser fileChooser = new JFileChooser();
                OpenFileExtensionFilter filter =
                    new OpenFileExtensionFilter (
                    ".ai0", ".ai", ".ai2", "JK/MotS AI File");

                fileChooser.addChoosableFileFilter (filter);
                fileChooser.setFileFilter (filter);
                fileChooser.showOpenDialog (null);

                if (GlobalConstants.VERBOSE_DEBUG)
                    System.out.println ("Opening File: " +
                    fileChooser.getSelectedFile() );

                try {
                    File fileName = fileChooser.getSelectedFile();

                    if (fileName == null)
                        return;

                    DataRecordOperations dataOpsEXTClass =
                        new DataRecordOperations();
                    dataOpsEXTClass.readFile (fileName);
                    codeWindow.setText (
                        dataOpsEXTClass.getTextDocument() );
                    codeWindow.setCaretPosition (0);

                    setStatusBarText (
                        "Last opened file: " +
                    	fileName.getCanonicalPath() );
                }
                catch (IOException ioErr) {
                    JOptionPane.showMessageDialog ( null,
                        resources.getProperty ("errorDesc11"),
                        resources.getProperty ("errorTitle11"),
                    JOptionPane.ERROR_MESSAGE);
                }
            }
            else
            if (source == parseExternalFile) {
                if (codeWindow.getText().equals ("") ) {
                    JOptionPane.showMessageDialog ( null,
                        resources.getProperty ("errorDesc12"),
                        resources.getProperty ("errorTitle12"),
                    JOptionPane.INFORMATION_MESSAGE);

                    // Don't proceed any further...
                }
                else {
                    try {
                        Parser parserEXTClass = new Parser();
                        parserEXTClass.parseDocument (codeWindow);

                        JOptionPane.showMessageDialog ( null,
                            resources.getProperty ("successDesc0"),
                            resources.getProperty ("successTitle0"),
                        JOptionPane.INFORMATION_MESSAGE);
                    }
                        // Stop being dependent on the resource file,
                        // just in case it goes missing/fails to load.
                        //
                        // In theory, this shouldn't happen, because
                        // the application should make sure it has all
                        // its resources available to load at startup
                        // (otherwise it exits), but better to be
                        // "safe than sorry"...
                        //
                    catch (java.io.FileNotFoundException ioFNF) {
                        JOptionPane.showMessageDialog ( null,
                            "An error has been encountered trying to locate " +
                            "the " + GlobalConstants.DATABASE_NAME +
                            "resource file. \n" +
                            "Please check that this file is present and " +
                            "hasn't been removed from the JAR-executable.",
                            "Error Parsing Document: Missing Necessary Resource File",
                        JOptionPane.ERROR_MESSAGE);
                    }
                    catch (java.io.IOException ioErr) {
                        JOptionPane.showMessageDialog ( null,
                            "An error has been encountered whilst " +
                            "attempting to load a required resource file. \n" +
                            "Please try the operation again.",
                            "Error Parsing Document: Could Not Load Resources",
                        JOptionPane.ERROR_MESSAGE);
                    }
                    catch (InstinctException instinct) {
                        JOptionPane.showMessageDialog ( null,
                            instinct.getMessage() ,
                            "Error Parsing Document: Instinct Error Encountered",
                        JOptionPane.WARNING_MESSAGE);
                    }
                }
            }
        }

    }   /* End Of "ButtonProcessor" Inner-Class   */


//=========================================================


    /**
     * This Inner-Class is responsible for over-all window behaviour.
     * <br /><br />
     * On the closing of the application, the screen location of the
     * application should be stored and any resources left-over
     * should be cleared up.
     * <br /><br />
     * On window closing, also calls the "Garbage Collector" before
     * exiting the JVM session.
     *
     */
    protected class WindowHandler extends WindowAdapter {

            /** The properties configuration file to access. */
        private Properties configurations = new Properties();
            /** The input stream to use as a source-loader for
                the properties configuration file. */
        private InputStream source;


        /**
         * Invoked when a window is in the process of being closed.
         * <br /><br />
         * Save window coordinates to central database file ready for
         * next time the application is loaded.
         *
         */
        public void windowClosing (WindowEvent event) {
            try {
                //source = getClass().getResourceAsStream (
                //    GlobalConstants.RESOURCE);
                source = new FileInputStream (GlobalConstants.RESOURCE);
                configurations.load (source);

                int temp;

                configurations.setProperty (
                    WINDOW_X_KEY,
                    Integer.toString(getX()) );

                configurations.setProperty (
                    WINDOW_Y_KEY,
                    Integer.toString (getY()) );

                configurations.setProperty (
                    WINDOW_WIDTH,
                    Integer.toString (getWidth()) );

                configurations.setProperty (
                    WINDOW_HEIGHT,
                    Integer.toString (getHeight()) );

                configurations.store (
                    new FileOutputStream (GlobalConstants.RESOURCE), null);

                if (GlobalConstants.VERBOSE_DEBUG) {
                    JOptionPane.showMessageDialog (null,
                        "Top Left: " +
                        configurations.getProperty ("top_left_coord") + "\n" +
                        "Bottom Left: " +
                        configurations.getProperty ("btm_left_coord") + "\n" +
                        "Top Right: " +
                        configurations.getProperty ("top_right_coord") + "\n" +
                        "Bottom Right: " +
                        configurations.getProperty ("btm_right_coord"),
                        "Database Resource Reference File Missing",
                        JOptionPane.ERROR_MESSAGE);
                }

                dispose();
                System.gc();
                System.exit (0);
            }
            catch (IOException IO) {
                JOptionPane.showMessageDialog (null,
                    "A critical application resource could not be located." +
                    "\n" +
                    "The application cannot function without it." +
                    "\n\n" +
                    "Could not find: " + configurations.toString() +
                    "\n\n" +
                    "Aborting . . . ",
                    "Database Resource Reference File Missing",
                    JOptionPane.ERROR_MESSAGE);
                System.gc();
                System.exit (1);
            }
        }
    }   /* End Of "WindowHandler" Inner-Class   */


//=========================================================


}   /* End Of File  */