

//  ######
//  AI Builder v2.0
//  Parser class.
//  By Richard John Sproston.
//  E-mail richard.sproston@gmail.com || lucky_jackpot@hotmail.com
//  Copyright () 2004/2005, Richard John Sproston.
//  All Rights Reserved.
//  Jan-2005
//  ######


package bin;


    // Import system class libaries for application
  import javax.swing.JTextArea;
  import javax.swing.text.Element;
  import java.util.*;
  import java.util.StringTokenizer;
  import java.util.Properties;
  import java.io.*;


/**
 * Parser Class
 * <p>
 * This class is used to parse any external files that are
 * loaded into AI Builder.
 * </p>
 * <b>NB:</b> As most of the code adheres to a particular format, the
 * documentation that accompanies the <code>parseIndividualInstinct()</code>
 * method is written out only once, in the first instinct
 * <code>("BlindFire")</code> code block.
 *
 * @author Richard John Sproston
 * [ <a href="mailto:richard.sproston@gmail.com">richard.sproston@gmail.com</a> ||
 * <a href="mailto:lucky_jackpot@hotmail.com">lucky_jackpot@hotmail.com</a> ]
 *
 * @version 0.1 - Initial Release
 *
 */
public class Parser {


//=========================================================


    //////////////////////////
    //  INSTANCE VARIABLES  //
    //////////////////////////


        /** The descriptor */
    private String paramTooltips = new String();
    private String currentLine;

        /** The particular instinct being parsed. */
    private String instinct;

        /** The number of parameters which each instinct uses. */
    private int numParams;

    private StringTokenizer token;

    // Database resource loading objects */
        /** The properties object that loads the input stream, referring
            to sirSource's source. */
    private Properties resources = new Properties();
        /** Stream-Input-Reader Source, required to load an input
            stream, referring to the resource file. */
    private InputStream sirSource;


//=========================================================


    ////////////////////
    //  CONSTRUCTORS  //
    ////////////////////


    /**
     * Default Constructor for class with no parameters.
     */
    public Parser()
    { }


    /**
     * Default Constructor for class with 1 parameter.
     *
     * @param instinct The current instinct being parsed.
     *
     */
    public Parser (String instinct) {
        this.instinct = instinct;
    }


    /**
     * Default Constructor for class with 2 parameters.
     *
     * @param instinct The current instinct being parsed.
     * @param numParams How many parameters the current instinct has.
     *
     */
    public Parser (String instinct, int numParams) {
        this.instinct = instinct;
        this.numParams = numParams;
    }


//=========================================================


    ////////////////////////
    //  ACCESS MODIFIERS  //
    ////////////////////////


    /**
     * Set the named contents of the current instinct.
     *
     * @param instinct The representative String name for
     *  the current instint.
     *
     */
    //public void setInstinct (String instinct) {
    //    this.instinct = instinct;
    //}


    /**
     * Set the number of parameters for the current instinct.
     *
     * @param numParams The number of parameters for
     *  the current instinct.
     *
     */
    //public void setNumParams (int numParams) {
    //    this.numParams = numParams;
    //}


//=========================================================


    /////////////////////////
    //  ACCESS RETRIEVERS  //
    /////////////////////////


    //public String getInstinct() {
    //    return instinct;
    //}


    //public int getNumParams() {
    //    return numParams;
    //}


    //public String getCurrentLine() {
    //    return currentLine;
    //}


//=========================================================


    ///////////////
    //  METHODS  //
    ///////////////


    /**
     * This method begins to parse an entire AI file document,
     * (as represented by the contents of the JTextArea "preview" pane.
     *
     * @param textArea The graphical component that contains the text
     *  that comprises the AI file's contents.
     *
     */
    public void parseDocument (JTextArea textArea)
        throws FileNotFoundException, IOException, InstinctException {

          // Get the text-area "paragraph" (content)
        Element paragraph = textArea.getDocument().getDefaultRootElement();

          // Get number of content elements
        int countContent = paragraph.getElementCount();

          // Get index ranges for each content element.
          // Each content element represents one line.
          // Each line includes the terminating newline (" \n ").
        for (int i=0 ; i<countContent ; i++) {
            Element e = paragraph.getElement(i);
            int rangeStart = e.getStartOffset();
            int rangeEnd = e.getEndOffset();

            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Start: " + rangeStart +
                "\t\t End: " + rangeEnd);


            try {
                String currentLine =
                    textArea.getText (rangeStart, rangeEnd - rangeStart);

                if (currentLine.startsWith (
                    GlobalConstants.COMMENT_IDENT) ) {

                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println ("Found: COMMENT on line: " +
                        (i+1));     // Offset, as 1st line = 0;
                }
                else
                if (rangeEnd == (rangeStart +1) ) {
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println ("Found a LINE-BREAK on line: " +
                        (i+1));     // Offset, as 1st line = 0;
                }
                else {
                    // We've found a valid AI code line if we're in this
                    // statement block...

                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println ("Found: CODE on line: " +
                        (i+1) );    // Offset, as 1st line = 0;

                        // ... now parse the (single) individual code line
                    //parseIndividualInstinct (currentLine, sir, (i+1) );
                    parseIndividualInstinct (currentLine, (i+1));
                }
            }
            catch (javax.swing.text.BadLocationException err) {
                if (GlobalConstants.VERBOSE_DEBUG)
                    System.out.println ("BadLocationException on file import");
            }
        }
    }






    /**
     * <p>
     * This method parses an individual line of code, attempting to locate
     * a specific instinct within the line, as well as checking parameter
     * count information as applicable to the instinct being parsed.
     * </p>
     * <p>
     * <b>NB:</b> The "Follow" instinct is separated into three sub-section
     * code blocks to evaluate its valid parameter range (this is because
     * "Follow" has three working implementations with 2, 3 and 4 parameter
     * versions (respectively) being shown to function correctly!
     * </p>
     *
     * @param individualInstinct The name of the specific instinct
     *  currently being parsed.
     * @param lineNumber The number of the line in the file where
     *  the instinct resides (used primarily to obtain a handle on
     *  line-parsing errors, so the user knows exactly <i>where</i>
     *  to look for the error).
     *
     * @throws FileNotFoundException If the <code>"ai.properties"</code>
     *  resource file (that acts as the program "database") cannot be
     *  located.
     * @throws IOException If any other possible I/O exception occurs
     *  whilst attempting to find/read a (core resource) file.
     * @throws InstinctException If an error parsing the currnet instinct
     *  occurs, prepare a handle to retrieve all related information
     *  (ready for a meaningful output display to the user).
     *
     */
    public void parseIndividualInstinct (
    String individualInstinct, int lineNumber)
        throws FileNotFoundException, IOException, InstinctException {

            // Load resources first!
        //sirSource = getClass().getResourceAsStream (GlobalConstants.RESOURCE);
        sirSource = new FileInputStream (GlobalConstants.RESOURCE);

            // Load the "ai.properties" file as the resources "database"
            // to cross-check instincts and parameters.
        resources.load (sirSource);

        int numParams;
        paramTooltips = new String();


            // Trim any white-space from beginning + end of lines...
        individualInstinct.trim();


        if (GlobalConstants.VERBOSE_DEBUG)
            resources.list (System.out);


        if (individualInstinct.startsWith ("BlindFire")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("BlindFire: " +
                resources.getProperty ("BlindFire"));

                // "substring()" the instinct to remove its name, leaving
                // just the comma-delimited numbers (as parameters).
                //
                // Split the String so that we just have the numbers
                // (ignoring text).  Then, "trim" leading & trailing spaces
                // from the String - contents in between are left alone!
            String instinctArgs = (individualInstinct.substring (9)).trim();

                // Set up a "StringTokenizer" for the instinct's parameters
                // (obtained by checking the resource's property key value),
                // checking for comma delimiters.
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("BlindFire")) ) {

                while (token.hasMoreTokens() ) {
                        // Some values are "floating-point", so parse
                        // them accordingly...
                    Double.parseDouble (token.nextToken() );

                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token.toString() );
                }
            }
            else {
                  // Create a new instance of the parsed error
                  // for this instinct...
                  //
                  //
                  // and "yes" - I am aware that this isn't the neatest
                  // layout ever for a "for" loop!
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("BlindFire")) ;
                    i++
                    )

                        // get a handle on how many parameters the instinct
                        // should take...
                    paramTooltips +=
                        resources.getProperty ("BlindFire" + i) + "\n";

                    // throw a customised InstinctException error, if
                    // the instinct has a particular error - either a
                    // miscount in parameter information or a typo-error...
                throw new InstinctException (
                    "BlindFire does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("BlindFire")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Charge")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Charge: " +
                resources.getProperty ("Charge") );

            String instinctArgs = (individualInstinct.substring (6)).trim();

            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Charge")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token.toString() );
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Charge")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Charge" + i) + "\n";

                throw new InstinctException (
                    "Charge does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Charge")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("CircleStrafe")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("CircleStrafe: " +
                resources.getProperty ("CircleStrafe") );

            String instinctArgs = (individualInstinct.substring (12)).trim();

            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("CircleStrafe")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token.toString() );
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("CircleStrafe")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("CircleStrafe" + i) + "\n";
                throw new InstinctException (
                    "CircleStrafe does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("CircleStrafe")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Crouch")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Crouch: " +
                resources.getProperty ("Crouch") );

            String instinctArgs = (individualInstinct.substring (6)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Crouch")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token.toString() );
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Crouch")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Crouch" + i) + "\n";

                throw new InstinctException (
                    "Crouch does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Crouch")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Dodge")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Dodge: " +
                resources.getProperty ("Dodge") );

            String instinctArgs = (individualInstinct.substring (5)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Dodge")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token.toString() );
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Dodge")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Dodge" + i) + "\n";

                throw new InstinctException (
                    "Dodge does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Dodge")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Flee")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Flee: " +
                resources.getProperty ("Flee") );

            String instinctArgs = (individualInstinct.substring (4)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Flee")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Flee")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Flee" + i) + "\n";

                throw new InstinctException (
                    "Flee does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Flee")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Follow")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Follow: " +
                resources.getProperty ("Follow2") );

            String instinctArgs = (individualInstinct.substring (6)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Follow2")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token.toString() );
                }
            }
            else
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Follow3") ) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token.toString() );
                }
            }
            else
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Follow4")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token.toString() );
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Follow4")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Follow4" + i) + "\n";

                throw new InstinctException (
                    "Follow does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Follow4")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("ForcePowers")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("ForcePowers: " +
                resources.getProperty ("ForcePowers") );

            String instinctArgs = (individualInstinct.substring (11)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("ForcePowers")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("ForcePowers")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("ForcePowers" + i) + "\n";

                throw new InstinctException (
                    "ForcePowers does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("ForcePowers")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("HitAndRun")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("HitAndRun: " +
                resources.getProperty ("HitAndRun") );

            String instinctArgs = (individualInstinct.substring (9)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("HitAndRun")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("HitAndRun")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("HitAndRun" + i) + "\n";

                throw new InstinctException (
                    "HitAndRun does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("HitAndRun")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Jump")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Jump: " +
                resources.getProperty ("Jump") );

            String instinctArgs = (individualInstinct.substring (4)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Jump")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Jump")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Jump" + i) + "\n";

                throw new InstinctException (
                    "Jump does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Jump")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Leap") ) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Leap: " +
                resources.getProperty ("Leap") );

            String instinctArgs = (individualInstinct.substring (4)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Leap")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Leap")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Leap" + i) + "\n";

                throw new InstinctException (
                    "Leap does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Leap")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Listen")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Listen: " +
                resources.getProperty ("Listen") );

            String instinctArgs = (individualInstinct.substring (6)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Listen")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Listen")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Listen" + i) + "\n";

                throw new InstinctException (
                    "Listen does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Listen") ),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("LobFire")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("LobFire: " +
                resources.getProperty ("LobFire") );

            String instinctArgs = (individualInstinct.substring (7)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("LobFire")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("LobFire")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("LobFire" + i) + "\n";

                throw new InstinctException (
                    "LobFire does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("LobFire")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("LookForTarget")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("LookForTarget: " +
                resources.getProperty ("LookForTarget") );

            String instinctArgs = (individualInstinct.substring (13)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("LookForTarget")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("LookForTarget")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("LookForTarget" + i) + "\n";

                throw new InstinctException (
                    "LookForTarget does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("LookForTarget") ),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("LookForOpposingTarget")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("LookForOpposingTarget: " +
                resources.getProperty ("LookForOpposingTarget") );

            String instinctArgs = (individualInstinct.substring (21)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("LookForOpposingTarget")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("LookForOpposingTarget")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("LookForOpposingTarget" + i) + "\n";

                throw new InstinctException (
                    "LookForOpposingTarget does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("LookForOpposingTarget")),
                    paramTooltips
                );
            }
        }
        //
        // OpenDoors is only here to comply with the parser's "else" statement
        // at the bottom of this method (otherwise it will throw the
        // exception, even if everything is valid!!!
        //
        else
        if (individualInstinct.startsWith ("OpenDoors")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("OpenDoors: " +
                resources.getProperty ("OpenDoors") );

            String instinctArgs = (individualInstinct.substring (9)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("OpenDoors")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("OpenDoors")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("OpenDoors" + i) + "\n";

                throw new InstinctException (
                    "OpenDoors does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("OpenDoors")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("PrimaryFire")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("PrimaryFire: " +
                resources.getProperty ("PrimaryFire") );

            String instinctArgs = (individualInstinct.substring (11)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("PrimaryFire")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("PrimaryFire")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("PrimaryFire" + i) + "\n";

                throw new InstinctException (
                    "PrimaryFire does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("PrimaryFire")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("RandomTurn")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("RandomTurn: " +
                resources.getProperty ("RandomTurn") );

            String instinctArgs = (individualInstinct.substring (10)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("RandomTurn")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("RandomTurn")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("RandomTurn" + i) + "\n";

                throw new InstinctException (
                    "RandomTurn does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("RandomTurn")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Retreat")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Retreat: " +
                resources.getProperty ("Retreat") );

            String instinctArgs = (individualInstinct.substring (7)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Retreat")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Retreat")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Retreat" + i) + "\n";

                throw new InstinctException (
                    "Retreat does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Retreat")),
                    paramTooltips
                );
            }
        }
        //
        // ReturnHome is only here to comply with the parser's "else" statement
        // at the bottom of this method (otherwise it will throw the
        // exception, even if everything is valid!!!
        //
        else
        if (individualInstinct.startsWith ("ReturnHome")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("ReturnHome: " +
                resources.getProperty ("ReturnHome") );

            String instinctArgs = (individualInstinct.substring (10)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("ReturnHome")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("ReturnHome")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("ReturnHome" + i) + "\n";

                throw new InstinctException (
                    "ReturnHome does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("ReturnHome")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Roam")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Roam: " +
                resources.getProperty ("Roam") );

            String instinctArgs = (individualInstinct.substring (4)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Roam")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Roam")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Roam" + i) + "\n";

                throw new InstinctException (
                    "Roam does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Roam")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("SaberFighting")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("SaberFighting: " +
                resources.getProperty ("SaberFighting") );

            String instinctArgs = (individualInstinct.substring (13)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("SaberFighting")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("SaberFighting")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("SaberFighting" + i) + "\n";

                throw new InstinctException (
                    "SaberFighting does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("SaberFighting")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("SenseDanger")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("SenseDanger: " +
                resources.getProperty ("SenseDanger") );

            String instinctArgs = (individualInstinct.substring (11)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("SenseDanger")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("SenseDanger")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("SenseDanger" + i) + "\n";

                throw new InstinctException (
                    "SenseDanger does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("SenseDanger")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("SpecialAttack")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("SpecialAttack: " +
                resources.getProperty ("SpecialAttack") );

            String instinctArgs = (individualInstinct.substring (13)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("SpecialAttack")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("SpecialAttack")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("SpecialAttack" + i) + "\n";

                throw new InstinctException (
                    "SpecialAttack does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("SpecialAttack")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Talk")) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Talk: " +
                resources.getProperty ("Talk") );

            String instinctArgs = (individualInstinct.substring (4)).trim();
            token = new StringTokenizer (instinctArgs, ",");

            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Talk")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                        System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Talk")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Talk" + i) + "\n";

                throw new InstinctException (
                    "Talk does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Talk") ),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("TurretFire") ) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("TurretFire: " +
                resources.getProperty ("TurretFire") );

            String instinctArgs = (individualInstinct.substring (10)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("TurretFire")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                    System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("TurretFire")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("TurretFire" + i) + "\n";

                throw new InstinctException (
                    "TurretFire does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("TurretFire")),
                    paramTooltips
                );
            }
        }
        else
        if (individualInstinct.startsWith ("Withdraw") ) {
            if (GlobalConstants.VERBOSE_DEBUG)
                System.out.println ("Withdraw: " +
                resources.getProperty ("Withdraw") );

            String instinctArgs = (individualInstinct.substring (10)).trim();
            token = new StringTokenizer (instinctArgs, ",");
            if (token.countTokens() ==
                Integer.parseInt (resources.getProperty ("Withdraw")) ) {

                while (token.hasMoreTokens() ) {
                    Double.parseDouble (token.nextToken() );
                    if (GlobalConstants.VERBOSE_DEBUG)
                    System.out.println (token);
                }
            }
            else {
                for (int i=0 ;
                    i<Integer.parseInt (resources.getProperty ("Withdraw")) ;
                    i++
                    )

                    paramTooltips +=
                        resources.getProperty ("Withdraw" + i) + "\n";

                throw new InstinctException (
                    "Withdraw does not have the correct parameters.",
                    lineNumber,
                    Integer.parseInt (resources.getProperty ("Withdraw")),
                    paramTooltips
                );
            }
        }
            // So all the available instincts have been dealt with...
            // ... now for the AI Setup Line...
        else
        if (individualInstinct.startsWith ("alignment") ) {
            System.out.println ("Located AI Setup Line");
        }
            // Lastly, if NOTHING else has fitted the criteria,
            // then the input (text in the preview "codeWindow")
            // must be fouled up/invalid...
        else {
            throw new InstinctException (
            "Preview window contains invalid or insufficient input.  \n" +
            "Unable to parse potential AI file! \n\n" +
            "It is possible that the instinct name has been spelt " +
            "incorrectly, as the error found was: \n" +
            individualInstinct, lineNumber);
        }

    }


//=========================================================


}   /* End Of File  */